unit XLCells;

interface
uses SysUtils, Windows, Messages, Classes, Controls, Graphics,
     StdCtrls, Forms, XLGrids, XLStream, ShellAPI,
     MaskUtils , Variants ;

type
  TXLCPicture = class(TXLCell)
  private
    FImage: TPicture;
    procedure ImageChanged(Sender: TObject);
  protected
    function  RecommendedSize(ACanvas: TCanvas): TPoint; override;
    function GetObject: TPersistent; override;
    procedure SetObject(AObject: TPersistent); override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
    procedure Draw(const ADraw: TXLDraw; var ARect: TRect); override;
    function HasImage: Boolean;
  public
    procedure Init; override;
    procedure Done; override;
  end;

  TXLCString = class(TXLCell)
  private
    FString: String;
  protected
    procedure SetString(const AValue: String); override;
    function  GetString: String; override;
    procedure SetValue(const AValue: Variant); override;
    function  GetValue: Variant; override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
    function FormatText: String; override;
  end;

  TXLCStrings = class(TXLCString)
  private
    FValue: Integer;
    FItems: TStringList;
  protected
    function  GetObject: TPersistent; override;
    procedure SetObject(AValue: TPersistent); override;
    function  GetInteger: Integer; override;
    procedure SetInteger(AValue: Integer); override;
    function  GetString: String; override;
    procedure SetString(const AValue: String); override;
    function  GetValue: Variant; override;
    procedure SetValue(const AValue: Variant); override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
    function FormatText: String; override;
  public
    procedure Init; override;
    procedure Done; override;
  end;

  TXLCNumber = class(TXLCell)
  protected
    function FormatText: String; override;
  end;

  TXLCInteger = class(TXLCNumber)
  private
    FValue: Integer;
  protected
    procedure SetInteger(AValue: Integer); override;
    function  GetInteger: Integer; override;
    procedure SetValue(const AValue: Variant); override;
    function  GetValue: Variant; override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
  end;

  TXLCDouble = class(TXLCNumber)
  private
    FValue: Double;
  protected
    procedure SetDouble(const AValue: Double); override;
    function  GetDouble: Double; override;
    procedure SetValue(const AValue: Variant); override;
    function  GetValue: Variant; override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
  end;

  TXLCCurrency = class(TXLCNumber)
  private
    FValue: Currency;
  protected
    procedure SetCurrency(const AValue: Currency); override;
    function  GetCurrency: Currency; override;
    procedure SetValue(const AValue: Variant); override;
    function  GetValue: Variant; override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
  end;

  TXLCDateTime = class(TXLCell)
  private
    FValue: TDateTime;
  protected
    procedure SetDateTime(const AValue: TDateTime); override;
    function  GetDateTime: TDateTime; override;
    procedure SetValue(const AValue: Variant); override;
    function  GetValue: Variant; override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
    function FormatText: String; override;
  end;

  TXLCBoolean = class(TXLCell)
  private
    FValue: Boolean;
  protected
    function FormatText: String; override;
    procedure SetBoolean(AValue: Boolean); override;
    function  GetBoolean: Boolean; override;
    procedure SetValue(const AValue: Variant); override;
    function  GetValue: Variant; override;
    procedure Load(AStream: TXLStream); override;
    procedure Save(AStream: TXLStream); override;
    procedure DblClick; override;
    procedure KeyPress(var Key: Char); override;
    procedure Draw(const ADraw: TXLDraw; var ARect: TRect); override;
  end;

  TXLCUrl = class(TXLCString)
    procedure DblClick; override;
  end;

  function FormatFloatEx(const Format: string; Value: Extended): string;

implementation uses Mask;

  function FormatFloatEx(const Format: string; Value: Extended): string;
  begin
    if Value > 0 then Result := UnpackPrefix(Format, ';')
    else begin
      Result := UnpackSuffix(Format, ';');
      if Result = '' then Result := UnpackPrefix(Format, ';')
      else Result := Format;
    end;
    if Pos('%', Result) > 0 then Value := Value * 100;
    Result := FormatFloat(Format, Value);
  end;

{ TXLCString }

function TXLCString.FormatText: String;
var
  DF: String;
begin
  DF := DisplayFormat;
  if DF > '' then Result := FormatMaskText(DF, FString)
  else Result := FString;
end;

function TXLCString.GetString: String;
begin
  Result := FString;
end;

function TXLCString.GetValue: Variant;
begin
  Result := FString;
end;

procedure TXLCString.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FString := AStream.StringIO;
end;

procedure TXLCString.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.StringIO := FString;
end;

procedure TXLCString.SetString(const AValue: String);
begin
  if FString <> AValue then
  begin
    FString := AValue;
    Changed;
  end;
end;

procedure TXLCString.SetValue(const AValue: Variant);
begin
  SetString(AValue);
end;

{ TXLCNumber }

function TXLCNumber.FormatText: String;
begin
  Result := FormatFloatEx(DisplayFormat, Value);
end;

{ TXLCInteger }

function TXLCInteger.GetInteger: Integer;
begin
  Result := FValue;
end;

function TXLCInteger.GetValue: Variant;
begin
  Result := FValue;
end;

procedure TXLCInteger.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FValue := AStream.IntegerIO;
end;

procedure TXLCInteger.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.IntegerIO := FValue;
end;

procedure TXLCInteger.SetInteger(AValue: Integer);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Changed;
  end;
end;

procedure TXLCInteger.SetValue(const AValue: Variant);
begin
  SetInteger(AValue);
end;

{ TXLCDouble }

function TXLCDouble.GetDouble: Double;
begin
  Result := FValue;
end;

function TXLCDouble.GetValue: Variant;
begin
  Result := FValue;
end;

procedure TXLCDouble.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FValue := AStream.DoubleIO;
end;

procedure TXLCDouble.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.DoubleIO := FValue;
end;

procedure TXLCDouble.SetDouble(const AValue: Double);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Changed;
  end;
end;

procedure TXLCDouble.SetValue(const AValue: Variant);
begin
  SetDouble(AValue);
end;

{ TXLCDateTime }

function TXLCDateTime.FormatText: String;
begin
  Result := FormatDateTime(DisplayFormat, FValue);
end;

function TXLCDateTime.GetDateTime: TDateTime;
begin
  Result := FValue;
end;

function TXLCDateTime.GetValue: Variant;
begin
  Result := FValue;
end;

procedure TXLCDateTime.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FValue := TDateTime(AStream.DoubleIO);
end;

procedure TXLCDateTime.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.DoubleIO := Double(FValue);
end;

procedure TXLCDateTime.SetDateTime(const AValue: TDateTime);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Changed;
  end;
end;

procedure TXLCDateTime.SetValue(const AValue: Variant);
begin
  if (VarType(AValue) and varString) <> 0 then begin
    try
      SetDateTime(StrToDateTime(AValue));
      exit;
    except
      {Invalid DateTime}
    end;
    try
      SetDateTime(StrToDate(AValue));
      exit;
    except
      {Invalid Date}
    end;
    try
      SetDateTime(StrToTime(AValue));
      exit;
    except
      {Invalid Time}
    end;
  end;
  SetDateTime(AValue);
end;

{ TXLCCurrency }

function TXLCCurrency.GetCurrency: Currency;
begin
  Result := FValue;
end;

function TXLCCurrency.GetValue: Variant;
begin
  Result := FValue;
end;

procedure TXLCCurrency.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FValue := Currency(AStream.DoubleIO);
end;

procedure TXLCCurrency.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.DoubleIO := Double(FValue);
end;

procedure TXLCCurrency.SetCurrency(const AValue: Currency);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Changed;
  end;
end;

procedure TXLCCurrency.SetValue(const AValue: Variant);
begin
  SetCurrency(AValue);
end;

function  AlignRect(const ARect: TRect; const ASize: TPoint; const ALayout: TXLLayout; AAllowStretch: Boolean): TRect;
var D1, D2: Double;
    X, Y: Integer;
    WM: TWrapMode;
begin
with ALayout do
begin
  WM := WrapMode;
  if not AAllowStretch then WM := wmClip;
  Result := ARect;
  case WM of
    wmClip: Result.BottomRight := Point(Result.Left + ASize.X, Result.Top + ASize.Y);
    wmEllipsis:
    begin
      D1 := (ARect.Right - ARect.Left) / ASize.X;
      D2 := (ARect.Bottom - ARect.Top) / ASize.Y;
      if D1 > D2 then
      begin
        Result := Bounds(ARect.Left, ARect.Top, Round(D2 * ASize.X), ARect.Bottom - ARect.Top);
        OffsetRect(Result, (ARect.Right - ARect.Left - Result.Right + Result.Left) div 2, 0);
      end else
      begin
        Result := Bounds(ARect.Left, ARect.Top, ARect.Right - ARect.Left, Round(D1 * ASize.Y));
        OffsetRect(Result, 0, (ARect.Bottom - ARect.Top - Result.Bottom + Result.Top) div 2);
      end;
    end;
  end;
  if WM <> wmWrapWords then
  begin
    case HorzAlign of
      haLeft  : X := ARect.Left - Result.Left;
      haRight : X := ARect.Right - Result.Right;
      else X := (ARect.Right + ARect.Left - (Result.Right - Result.Left)) div 2 - Result.Left;
    end;
    case VertAlign of
      vaTop   : Y := ARect.Top - Result.Top;
      vaBottom: Y := ARect.Bottom - Result.Bottom;
      else Y := (ARect.Bottom + ARect.Top - (Result.Bottom - Result.Top)) div 2 - Result.Top;
    end;
    OffsetRect(Result, X, Y);
  end;
end;
end;

{ TXLCBoolean }

procedure TXLCBoolean.DblClick;
begin
  AsBoolean := not AsBoolean;
end;

function TXLCBoolean.FormatText: String;
begin
  if FValue then Result := UnpackPrefix(Format, ';')
  else Result := UnpackSuffix(Format, ';');
end;

procedure TXLCBoolean.Draw(const ADraw: TXLDraw; var ARect: TRect);
var F: Integer;
    R: TRect;
begin
  if Format > '' then inherited Draw(ADraw, ARect) else begin
    R := AlignRect(ARect, Point(13 * ADraw.Scale div 100, 13 * ADraw.Scale div 100), ADraw.Layout, False);
    F := DFCS_BUTTONCHECK or DFCS_FLAT;
    if FValue then F := F or DFCS_CHECKED;
    DrawFrameControl(ADraw.Canvas.Handle, R, DFC_BUTTON, F);
  end;
end;

function TXLCBoolean.GetBoolean: Boolean;
begin
  Result := FValue;
end;

function TXLCBoolean.GetValue: Variant;
begin
  Result := FValue;
end;

procedure TXLCBoolean.KeyPress(var Key: Char);
begin
  if Key = #32 then
  begin
    DblClick;
    Key := #0;
  end;
end;

procedure TXLCBoolean.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FValue := Boolean(AStream.IntegerIO);
end;

procedure TXLCBoolean.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.IntegerIO := Integer(FValue);
end;

procedure TXLCBoolean.SetBoolean(AValue: Boolean);
begin
  if AValue <> FValue then
  begin
    FValue := AValue;
    Changed;
  end;
end;

procedure TXLCBoolean.SetValue(const AValue: Variant);
begin
  SetBoolean(AValue);
end;

{ TXLFiler }

type
  TXLFiler = class(TFiler)
  private
    FRead: Boolean;
    FStream: TXLStream;
  public
    procedure DefineBinaryProperty(const Name: string; ReadData, WriteData: TStreamProc; HasData: Boolean); override;
    procedure DefineProperty(const Name: string; ReadData: TReaderProc; WriteData: TWriterProc; HasData: Boolean); override;
    procedure FlushBuffer; override;
    constructor Create(AStream: TXLStream; ARead: Boolean);
  end;

constructor TXLFiler.Create(AStream: TXLStream; ARead: Boolean);
begin
  inherited Create(AStream, 4096);
  FStream := AStream;
  FRead := ARead;
end;

procedure TXLFiler.DefineBinaryProperty(const Name: string; ReadData,
  WriteData: TStreamProc; HasData: Boolean);
begin
  if FRead then
  begin
    FStream.Read(HasData, SizeOf(Boolean));
    if HasData then ReadData(FStream);
  end else
  begin
    FStream.Write(HasData, SizeOf(Boolean));
    if HasData then WriteData(FStream);
  end;
end;

procedure TXLFiler.DefineProperty(const Name: string;
  ReadData: TReaderProc; WriteData: TWriterProc; HasData: Boolean);
begin
  {No support for non-binary props}
end;

procedure TXLFiler.FlushBuffer;
begin
  {Do Nothing}
end;

{ TXLPicture }

type
  TXLPicture = class(TPicture)
    procedure Serialize(Stream: TXLStream; ARead: Boolean);
  end;

procedure TXLPicture.Serialize(Stream: TXLStream; ARead: Boolean);
var R: TXLFiler;
begin
  R := TXLFiler.Create(Stream, ARead);
  try
    DefineProperties(R);
  finally
    R.Free;
  end;
end;

{ TXLCPicture }

procedure TXLCPicture.Init;
begin
  FImage := TXLPicture.Create;
  FImage.OnChange := ImageChanged;
end;

procedure TXLCPicture.Done;
begin
  FImage.Free;
end;

procedure TXLCPicture.ImageChanged(Sender: TObject);
begin
  Changed;
end;

function TXLCPicture.GetObject: TPersistent;
begin
  Result := FImage;
end;

procedure TXLCPicture.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  TXLPicture(FImage).Serialize(AStream, True);
end;

procedure TXLCPicture.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  TXLPicture(FImage).Serialize(AStream, False);
end;

procedure TXLCPicture.SetObject(AObject: TPersistent);
begin
  FImage.Assign(AObject);
end;

function   TXLCPicture.HasImage: Boolean;
begin
  Result := ((FImage.Graphic <> Nil) and not FImage.Graphic.Empty);
end;

procedure  TXLCPicture.Draw(const ADraw: TXLDraw; var ARect: TRect);
begin
  if HasImage
  then ADraw.Canvas.StretchDraw(AlignRect(ARect, Point(FImage.Width * ADraw.Scale div 100, FImage.Height * ADraw.Scale div 100), ADraw.Layout, not (FImage.Graphic is TIcon)), FImage.Graphic);
end;

function TXLCPicture.RecommendedSize(ACanvas: TCanvas): TPoint;
begin
  Result := Point(0, 0);
  if HasImage and (WrapMode = wmClip) then Result := Point(FImage.Graphic.Width + Indent, FImage.Graphic.Height);
end;

{ TXLCStrings }

procedure TXLCStrings.Init;
begin
  FItems := TStringList.Create;
end;

procedure TXLCStrings.Done;
begin
  FItems.Free;
end;

function TXLCStrings.FormatText: String;
var
  DF: String;
begin
  Result := '';
  if (FValue < FItems.Count) and (FValue >= 0) then Result := FItems[FValue];
  DF := DisplayFormat;
  if DF > '' then Result := FormatMaskText(DF, Result);
end;

function TXLCStrings.GetInteger: Integer;
begin
  Result := FValue;
end;

function TXLCStrings.GetObject: TPersistent;
begin
  Result := FItems;
end;

function TXLCStrings.GetString: String;
begin
  Result := FItems.Text;
end;

function TXLCStrings.GetValue: Variant;
begin
  Result := GetString;
end;

procedure TXLCStrings.Load(AStream: TXLStream);
begin
  inherited Load(AStream);
  FValue := AStream.IntegerIO;
  FItems.Text := AStream.StringIO;
end;

procedure TXLCStrings.Save(AStream: TXLStream);
begin
  inherited Save(AStream);
  AStream.IntegerIO := FValue;
  AStream.StringIO := FItems.Text;
end;

procedure TXLCStrings.SetInteger(AValue: Integer);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Changed;
  end;
end;

procedure TXLCStrings.SetObject(AValue: TPersistent);
begin
  FItems.Assign(AValue);
  Changed;
end;

procedure TXLCStrings.SetString(const AValue: String);
begin
  if AValue <> GetString then
  begin
    FItems.Text := AValue;
    Changed;
  end;
end;

procedure TXLCStrings.SetValue(const AValue: Variant);
begin
  SetString(AValue);
end;

{ TXLCUrl }

procedure TXLCUrl.DblClick;
begin
  ShellExecute(Application.Handle, nil, PChar(AsString), Nil, Nil, SW_SHOWNOACTIVATE);
end;

initialization
  with XLManager do begin
    RegisterCellClasses([TXLCString, TXLCStrings, TXLCInteger, TXLCDouble, TXLCCurrency, TXLCDateTime, TXLCBoolean, TXLCPicture, TXLCUrl]);
    with Formats[TXLCString] do begin
      Add('');
    end;
    with Formats[TXLCInteger] do begin
      Add('0');
      Add('0|spin');
    end;
    with Formats[TXLCDouble] do begin
      Add('0');
      Add('0|spin');
      Add('0.00');
      Add('0.00|spin');
      Add('#,##0');
      Add('#,##0|spin');
      Add('#,##0.00');
      Add('#,##0.00|spin');
      Add('0.##%');
      Add('0.##%|spin');
      Add('0.00%');
      Add('0.00%|spin');
    end;
    with Formats[TXLCCurrency] do begin
      Add('$# ##0');
      Add('$# ##0|spin');
      Add('$# ##0.00');
      Add('$# ##0.00|spin');
    end;
    with Formats[TXLCDateTime] do begin
      Add('DD.MM.YYYY');
      Add('DD.MM.YY');
      Add('DD.MM');
      Add('MM.YY');
      Add('HH.MM.SS');
      Add('h:nn AM/PM');
      Add('h:nn:ss AM/PM');
      Add('h:nn');
      Add('h:nn:ss');
      Add('DD.MM.YYYY h:nn');
    end;
    Formats[TXLCBoolean].Add('True;False');
  end;
end.


