unit XLColors;

{$I Defines.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Menus, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls;

type
  TColorToolButton = class(TToolButton)
  private
    FOnDropDown: TNotifyEvent;
{$IFNDEF Delphi45}
    FPopup: TPopupMenu;
  protected
    procedure Loaded; override;
{$ENDIF}
  protected
    procedure CMColorChanged(var Msg: TMessage); message CM_COLORCHANGED;
    procedure Paint; override;
  public
{$IFDEF Delphi45}
    function CheckMenuDropdown: Boolean; override;
{$ENDIF}
    constructor Create(AOwner: TComponent); override;
  published
    property DropdownMenu stored False;
    property OnDropDown: TNotifyEvent read FOnDropDown write FOnDropDown;
    property Color default clBlack;
  end;

  TColors = array[0..55] of TColor;

  TXLColorBox = class(TGraphicControl)
  private
    FGlobal: Boolean;
    FColors: TColors;
    FAuto: Boolean;
    FActive: Integer;
    FOnChange: TNotifyEvent;
    FItemHeight: integer;
    FTabOrder: integer;
    procedure SetColors(AIndex: Integer; AColor: TColor);
    function  GetColors(AIndex: Integer): TColor;
    procedure SetActive(AIndex: Integer);
    function  GetColor: TColor;
    procedure SetColor(AColor: TColor);
    procedure SetGlobal(Value: Boolean);
    procedure SetAuto(Value: Boolean);
    function AutoSize: Integer;
    procedure SetItemHeight(const Value: integer);
    procedure SetTabOrder(const Value: integer);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  public
    function  HitTest(X, Y: Integer): Integer;
    constructor Create(AOwner: TComponent); override;
    property  Colors[AIndex: Integer]: TColor read GetColors write SetColors;
    property  ActiveColor: TColor read GetColor write SetColor;
  published
    property  ShowAuto: Boolean read FAuto write SetAuto default True;
    property  Global: Boolean read FGlobal write SetGlobal default True;
    property  OnClick;
    property  OnDblClick;
    property  ActiveIndex: Integer read FActive write SetActive default -1;
    property  OnChange: TNotifyEvent read FOnChange write FOnChange;
    property  Width default 18*8;
    property  Height default 19*7 + 20;
    property  Align;
    property  ItemHeight:integer read FItemHeight write SetItemHeight;
    property  TabOrder:integer read FTabOrder write SetTabOrder;
  end;

  TXLPopupPanel = class(TCustomControl)
  private
    FPopup: TWinControl;
    FOnClose: TNotifyEvent;
    FCaller: TControl;
    function GetActive: Boolean;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Popup(ACaller: TControl; AAlign: TAlignment);
    procedure Close;
    property  Caller: TControl read FCaller;
    property  Active: Boolean read GetActive;
  published
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
  end;
  procedure Register;
var GlobalColors: TColors;

implementation

procedure CreateGlobalColors;
begin
  GlobalColors[0] := $0;
  GlobalColors[1] := RGB(146, 48, 0);
  GlobalColors[2] := RGB(48, 48, 0);
  GlobalColors[3] := RGB(0, 48, 0);
  GlobalColors[4] := RGB(0, 48, 96);
  GlobalColors[5] := RGB(0, 0, 127);
  GlobalColors[6] := RGB(48, 48, 146);
  GlobalColors[7] := RGB(48, 48, 48);

  GlobalColors[8] := RGB(127, 0, 0);
  GlobalColors[9] := RGB(255, 96, 0);
  GlobalColors[10] := RGB(127, 127, 0);
  GlobalColors[11] := RGB(0, 127, 0);
  GlobalColors[12] := RGB(0, 127, 127);
  GlobalColors[13] := RGB(0, 0, 255);
  GlobalColors[14] := RGB(96, 96, 146);
  GlobalColors[15] := RGB(127, 127, 127);

  GlobalColors[16] := RGB(255, 0, 0);
  GlobalColors[17] := RGB(255, 146, 0);
  GlobalColors[18] := RGB(146, 200, 0);
  GlobalColors[19] := RGB(48, 151, 96);
  GlobalColors[20] := RGB(48, 200, 200);
  GlobalColors[21] := RGB(48, 96, 255);
  GlobalColors[22] := RGB(127, 0, 127);
  GlobalColors[23] := RGB(146, 146, 146);

  GlobalColors[24] := RGB(255, 0, 255);
  GlobalColors[25] := RGB(255, 200, 0);
  GlobalColors[26] := RGB(255, 255, 0);
  GlobalColors[27] := RGB(0, 255, 0);
  GlobalColors[28] := RGB(0, 255, 255);
  GlobalColors[29] := RGB(0, 200, 255);
  GlobalColors[30] := RGB(146, 48, 96);
  GlobalColors[31] := RGB(191, 191, 191);

  GlobalColors[32] := RGB(255, 151, 200);
  GlobalColors[33] := RGB(255, 200, 146);
  GlobalColors[34] := RGB(255, 255, 146);
  GlobalColors[35] := RGB(200, 255, 200);
  GlobalColors[36] := RGB(200, 255, 255);
  GlobalColors[37] := RGB(146, 200, 255);
  GlobalColors[38] := RGB(200, 151, 255);
  GlobalColors[39] := RGB(255, 255, 255);

  GlobalColors[40] := RGB(146, 151, 255);
  GlobalColors[41] := RGB(146, 48, 96);
  GlobalColors[42] := RGB(255, 255, 200);
  GlobalColors[43] := RGB(200, 255, 255);
  GlobalColors[44] := RGB(96, 0, 96);
  GlobalColors[45] := RGB(255, 127, 127);
  GlobalColors[46] := RGB(0, 100, 200);
  GlobalColors[47] := RGB(200, 200, 255);

  GlobalColors[48] := RGB(0, 0, 127);
  GlobalColors[49] := RGB(255, 0, 255);
  GlobalColors[50] := RGB(255, 255, 0);
  GlobalColors[51] := RGB(0, 255, 255);
  GlobalColors[52] := RGB(127, 0, 127);
  GlobalColors[53] := RGB(127, 0, 0);
  GlobalColors[54] := RGB(0, 127, 127);
  GlobalColors[55] := RGB(0, 0, 255);
end;

{ TColorToolButton }
{$IFDEF Delphi45}
function TColorToolButton.CheckMenuDropdown: Boolean;
begin
  if Assigned(FOnDropDown) then FOnDropDown(Self);
  Result := True;
end;

constructor TColorToolButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ParentColor := False;
  Color := clBlack;
end;

{$ELSE}
type
  TFakePopupMenu = class(TPopupMenu)
    procedure Popup(X, Y: Integer); override;
  end;

  procedure TFakePopupMenu.Popup(X, Y: Integer);
  begin
    with TColorToolButton(PopupComponent) do begin
      if Assigned(FOnDropDown)
      then FOnDropDown(PopupComponent);
    end;
  end;

constructor TColorToolButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPopup := TFakePopupMenu.Create(Self);
  ParentColor := False;
  Color := clBlack;
end;

procedure TColorToolButton.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then DropDownMenu := FPopup;
end;

{$ENDIF}

procedure TColorToolButton.CMColorChanged(var Msg: TMessage);
begin
  Invalidate;
end;

procedure TColorToolButton.Paint;
begin
  inherited Paint;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(Rect(3, Height - 5, 19, Height - 2));
end;

{ TXLColorBox }

constructor TXLColorBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  Width := 18*8;
  Height := 19*7 + 20;
  FActive := -1;
  FAuto := True;
  Global := True;
end;

function TXLColorBox.GetColor: TColor;
begin
  Result := Colors[FActive];
end;

function TXLColorBox.GetColors(AIndex: Integer): TColor;
begin
  Result := clNone;
  if (AIndex >= 0) and (AIndex <= 55) then
    if FGlobal then Result := GlobalColors[AIndex]
    else Result := FColors[AIndex];
end;

function TXLColorBox.HitTest(X, Y: Integer): Integer;
var CSX, CSY,
    CI: Integer;
    R: TRect;

function BlockHitTest(YCount: Integer): Integer;
var xX, yY: Integer;
begin
  for yY := 0 to YCount do begin
    OffsetRect(R, -R.Left, 0);
    for xX := 0 to 7 do begin
      if PtInRect(R, Point(X, Y)) then begin
        Result := CI;
        exit;
      end;
      Inc(CI);
      OffsetRect(R, CSX, 0);
    end;
    OffsetRect(R, 0, CSY);
  end;
  Result := -2;
end;

var
  AdjHeight: Integer;

begin
  Result := -1;
  AdjHeight := ClientHeight;
  if ShowAuto then begin
    if Y < AutoSize then exit;
    Dec(Y, AutoSize);
    Dec(AdjHeight, AutoSize);
  end;
  CSX := ClientWidth div 8;
  CSY := (AdjHeight - 7) div 7;
  R := Bounds(0, 0, CSX, CSY);
  CI := 0;
  Result := BlockHitTest(4);
  if Result < 0 then begin
    OffsetRect(R, 0, 7);
    Result := BlockHitTest(1);
  end;
end;

procedure TXLColorBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  ActiveIndex := HitTest(X, Y);
end;

procedure TXLColorBox.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if csLButtonDown in ControlState
  then ActiveIndex := HitTest(X, Y);
  inherited MouseMove(Shift, X, Y);
end;

procedure TXLColorBox.Paint;
var CSX, CSY,
    CI, LY: Integer;
    R: TRect;
    TopOffset: Integer;
    AdjHeight: Integer;

procedure DrawAuto;
var
  R: TRect;
begin
  R := Rect(0, 0, CSX * 8, TopOffset);
  InflateRect(R, -2, -2);
  if ActiveIndex = -1
  then DrawFrameControl(Canvas.Handle, R, DFC_BUTTON, DFCS_BUTTONPUSH or DFCS_CHECKED or DFCS_PUSHED)
  else DrawEdge(Canvas.Handle, R, BDR_RAISEDOUTER, BF_RECT or BF_FLAT or BF_MIDDLE);
  SetBkMode(Canvas.Handle, TRANSPARENT);
  DrawText(Canvas.Handle, 'Auto', 4, R, DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  ExcludeClipRect(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom);
end;

procedure DrawColors(YCount: Integer);
var X, Y: Integer;
    R1: TRect;
begin
  for Y := 0 to YCount do begin
    OffsetRect(R, -R.Left, 0);
    for X := 0 to 7 do begin
      R1 := R;
      InflateRect(R1, -4, -4);
      Canvas.Brush.Color := Colors[CI];
      Canvas.FillRect(R1);
      InflateRect(R1, 1, 1);
      DrawEdge(Canvas.Handle, R1, BDR_RAISEDOUTER, BF_RECT or BF_FLAT);
      ExcludeClipRect(Canvas.Handle, R1.Left, R1.Top, R1.Right, R1.Bottom);
      if CI = ActiveIndex then begin
        DrawFrameControl(Canvas.Handle, R, DFC_BUTTON, DFCS_BUTTONPUSH or DFCS_CHECKED or DFCS_PUSHED);
        ExcludeClipRect(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom);
      end;
      Inc(CI);
      OffsetRect(R, CSX, 0);
    end;
    OffsetRect(R, 0, CSY);
  end;
end;

begin
  with Canvas do begin
    AdjHeight := ClientHeight;
    CSX := ClientWidth div 8;
    TopOffset := 0;
    if ShowAuto then begin
      TopOffset := AutoSize;
      Dec(AdjHeight, TopOffset);
      DrawAuto;
    end;
    CSY := (AdjHeight - 6) div 7;
    R := Bounds(0, TopOffset, CSX, CSY);
    CI := 0;
    Pen.Width := 1;
    Pen.Color := clBtnShadow;
    Brush.Style := bsSolid;
    DrawColors(4);
    LY := R.Top + 3;
    OffsetRect(R, 0, 7);
    DrawColors(1);
    Brush.Color := Color;
    FillRect(ClientRect);
    MoveTo(2, LY);
    LineTo(CSX * 8 - 2, LY);
    Inc(LY);
    Pen.Color := clBtnHighlight;
    MoveTo(2, LY);
    LineTo(CSX * 8 - 2, LY);
  end;
end;

procedure TXLColorBox.SetActive(AIndex: Integer);
begin
  if AIndex < -1 then exit;
  if (AIndex > 55) then AIndex := -1;
  if AIndex <> FActive then
  begin
    FActive := AIndex;
    if Assigned(FOnChange) then FOnChange(Self);
    Invalidate;
  end;
end;

function TXLColorBox.AutoSize: Integer;
begin
  Result := Abs(Font.Height) + 9;
end;

procedure TXLColorBox.SetAuto(Value: Boolean);
begin
  if ShowAuto <> Value then begin
    FAuto := Value;
    if not (csLoading in ComponentState) then begin
      if ShowAuto then Height := Height + AutoSize
      else Height := Height - AutoSize;
      Invalidate;
    end;
  end;
end;

procedure TXLColorBox.SetColor(AColor: TColor);
var I: Integer;
begin
  for I := 0 to 55 do
  if FColors[I] = AColor then begin
    ActiveIndex := I;
    exit;
  end;
  ActiveIndex := -1;
end;

procedure TXLColorBox.SetColors(AIndex: Integer; AColor: TColor);
begin
  if (AIndex >= 0) or (AIndex <= 55) then begin
    FColors[AIndex] := AColor;
    if FGlobal then GlobalColors[AIndex] := AColor;
    Invalidate;
  end;
end;

procedure TXLColorBox.SetGlobal(Value: Boolean);
begin
  FGlobal := Value;
  if FGlobal then FColors := GlobalColors;
  Invalidate;
end;


{ TPopup }
const
  CM_KILL = WM_USER + $4321;

type

  TPopup = class(TWinControl)
  private
    FPanel: TXLPopupPanel;
    FCaller: TControl;
    FOnClose: TNotifyEvent;
    procedure MoveTo(ASource, ADest: TWinControl);
    procedure CloseControl;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WMKillFocus(var Msg: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMCancelMode(var Msg: TWMCancelMode); message WM_CANCELMODE;
    procedure CMKill(var Msg: TMessage); message CM_KILL;
  public
    constructor Popup(APanel: TXLPopupPanel; ACaller: TControl; AAlign: TAlignment);
    procedure Close;
    property  Caller: TControl read FCaller;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
  end;

procedure TPopup.MoveTo(ASource, ADest: TWinControl);
var
  I: Integer;
begin
  for I := ASource.ControlCount - 1 downto 0 do ASource.Controls[I].Parent := ADest;
end;

procedure TPopup.CloseControl;
begin
  if Caller <> Nil then begin
    FCaller := Nil;
    Visible := False;
    MoveTo(Self, FPanel);
    FPanel.FPopup := Nil;
    if Assigned(FOnClose) then FOnClose(FPanel);
    FPanel := Nil;
    Free;
  end;
end;

procedure TPopup.CMKill(var Msg: TMessage);
begin
  CloseControl;
end;

procedure TPopup.Close;
begin
  if HandleAllocated then PostMessage(Handle, CM_KILL, 0, 0)
  else CloseControl;
end;

procedure TPopup.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := WS_POPUP;
  Params.ExStyle := WS_EX_PALETTEWINDOW or WS_EX_DLGMODALFRAME or WS_EX_NOPARENTNOTIFY;
end;

constructor TPopup.Popup(APanel: TXLPopupPanel; ACaller: TControl; AAlign: TAlignment);
var PO, PS: TPoint;
    X: Integer;
begin
  inherited Create(nil);
  Visible := False;
  Parent := APanel.Parent;
  FPanel := APanel;
  FCaller := ACaller;
  Width := FPanel.Width;
  Height := FPanel.Height;
  MoveTo(FPanel, Self);
  PO := Caller.ClientRect.BottomRight;
  PS := Caller.ClientToScreen(PO);
  case AAlign of
   taLeftJustify: X := PS.X - PO.X;
   taRightJustify: X := PS.X - Width;
   else X := PS.X - (Width - PO.X) div 2;
  end;
  SetBounds(X, PS.Y, Width, Height);
  Visible := True;
  Windows.SetFocus(Handle);
end;

procedure TPopup.WMCancelMode(var Msg: TWMCancelMode);
begin
  inherited;
  Close;
end;

procedure TPopup.WMKillFocus(var Msg: TWMKillFocus);
begin
  inherited;
  if Msg.FocusedWnd <> Handle then Close;
end;

procedure TXLColorBox.SetItemHeight(const Value: integer);
begin
  FItemHeight := Value;
end;

procedure TXLColorBox.SetTabOrder(const Value: integer);
begin
  FTabOrder := Value;
end;

{ TXLPopupPanel }

procedure TXLPopupPanel.Close;
begin
  if FPopup <> Nil then TPopup(FPopup).Close;
end;

constructor TXLPopupPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Visible := False;
  ControlStyle := ControlStyle + [csAcceptsControls];
end;

procedure TXLPopupPanel.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := WS_EX_DLGMODALFRAME;
end;

function TXLPopupPanel.GetActive: Boolean;
begin
  Result := FPopup <> Nil;
end;

procedure TXLPopupPanel.Popup(ACaller: TControl; AAlign: TAlignment);
begin
  FCaller := ACaller;
  FPopup := TPopup.Popup(Self, ACaller, AAlign);
  TPopup(FPopup).OnClose := FOnClose;
end;
procedure Register;
begin
  RegisterComponents('XLGrids', [ TColorToolButton, TXLColorBox, TXLPopupPanel]);
end;


initialization
  CreateGlobalColors;

end.
