unit XLEdits;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ComCtrls,
  XLGrids, XLLists, XLCells, Mask;

const
  WM_PULATTACH = WM_APP + $765;

type
  TPickList = class(TCustomComboBox)
  private
    FIndent: Integer;
    FAlign: TAlignment;
    procedure SendParams;
    procedure SetIndent(Value: Integer);
    procedure SetAlign(Value: TAlignment);
  protected
    procedure Change; dynamic; // known warning
    procedure CreateWnd; override;
    procedure WMPULAttach(var Msg: TMessage); message WM_PULATTACH;
    procedure KeyPress(var Key: Char); override;
  public
    constructor Create(AOwner: TComponent); override;
    property Indent: Integer read FIndent write SetIndent;
    property Alignment: TAlignment read FAlign write SetAlign default taLeftJustify;
    property EditHandle;
    property ParentColor;
    property ParentFont;
    property MaxLength;
  end;

  TXLMemo = class(TMemo)
  private
    FIndent: Integer;
    procedure XLLayout(var Msg: TXLCellLayout); message XL_LAYOUT;
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
    procedure XLSaveCell(var Msg: TXLEditCell); message XL_SAVECELL;
    procedure WMSetFont(var Message: TWMSetFont); message WM_SETFONT;
    procedure SetIndent(Value: Integer);
    procedure UpdateHeight;
  protected
    procedure Change; override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property BorderStyle default bsNone;
    property Indent: Integer read FIndent write SetIndent;
  end;

  TXLMaskEdit = class(TMaskEdit)
  private
    FInvalid: Boolean;
    FIndent: Integer;
    FAlign: TAlignment;
    procedure SetAlign(Value: TAlignment);
    procedure CMExit(var Msg: TMessage); message CM_EXIT;
    procedure CNKeyDown(var Msg: TWMKeyDown); message CN_KEYDOWN;
    procedure XLLayout(var Msg: TXLCellLayout); message XL_LAYOUT;
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
    procedure XLSaveCell(var Msg: TXLEditCell); message XL_SAVECELL;
    procedure WMSetFont(var Message: TWMSetFont); message WM_SETFONT;
    procedure SetIndent(const Value: Integer);
    procedure UpdateHeight;
  protected
    procedure UpdateParams; virtual;
    procedure Change; override;
    procedure CreateWnd; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure PostValidate; dynamic;
  public
    property Invalid: Boolean read FInvalid;
    constructor Create(AOwner: TComponent); override;
  published
    property Indent: Integer read FIndent write SetIndent;
    property Alignment: TAlignment read FAlign write SetAlign default taLeftJustify;
    property BorderStyle default bsNone;
  end;

  TXLNumEdit = class(TXLMaskEdit)
  private
    FValue,
    FValueMin,
    FValueMax: Double;
    FLocked: Boolean;
    FFormat: String;
    FIntMode: Boolean;
    FInc: Double;
    FBar: TScrollBar;
    procedure OnScroll(Sender: TObject);
    procedure SetMax(const AMax: Double);
    procedure SetMin(const AMin: Double);
    procedure SetValue(const AFloat: Double);
    procedure UpdateText;
    function  UseRange: Boolean;
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
    procedure XLSaveCell(var Msg: TXLEditCell); message XL_SAVECELL;
    function  GetValue: Double;
    function GetSpinner: Boolean;
    procedure SetSpinner(const Value: Boolean);
  protected
    procedure UpdateParams; override;
    procedure SetFormat(const AFormat: String); 
    procedure KeyPress(var Key: Char); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure PostValidate; override;
  protected
    function  IsValidChar(Key: Char): Boolean; virtual;
    procedure AlignControls(AControl: TControl; var ARect: TRect); override;
  public
    constructor Create(AOwner: TComponent); override;
    property IntMode: Boolean read FIntMode;
  published
    property Value: Double read GetValue write SetValue;
    property Format: String read FFormat write SetFormat;
    property MaxValue: Double read FValueMax write SetMax;
    property MinValue: Double read FValueMin write SetMin;
    property Spinner: Boolean read GetSpinner write SetSpinner;
    property Increment: Double read FInc write FInc;
  end;

  TXLPickStyle = (psDropDown, psDropDownList);

  TXLPickEdit = class(TWinControl)
  private
    FPickList: TPickList;
    function GetItem: Integer;
    function GetItems: TStrings;
    function GetText: String;
    procedure SetItem(Value: Integer);
    procedure SetItems(Value: TStrings);
    procedure SetText(const Value: String);
    function GetDroppedDown: Boolean;
    procedure SetDroppedDown(Value: Boolean);
    function GetMaxLength: Integer;
    procedure SetMaxLength(Value: Integer);
    function GetSorted: Boolean;
    procedure SetSorted(Value: Boolean);
    function GetOnChange: TNotifyEvent;
    procedure SetOnChange(Value: TNotifyEvent);
    function GetOnDropDown: TNotifyEvent;
    procedure SetOnDropDown(Value: TNotifyEvent);
    procedure AddToList(ACell: TXLCell; AData: Integer; var Stop: Boolean);
  protected
    function AdjustHeight: Integer; virtual;
    procedure AlignControls(AControl: TControl; var ARect: TRect); override;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure XLEditInfo(var Msg: TXLEditInfo); message XL_EDITINFO;
    procedure XLLayout(var Msg: TXLCellLayout); message XL_LAYOUT;
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
    procedure XLSaveCell(var Msg: TXLEditCell); message XL_SAVECELL;
  public
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(AOwner: TComponent); override;
    property DroppedDown: Boolean read GetDroppedDown write SetDroppedDown;
  published
    property Items: TStrings read GetItems write SetItems;
    property ItemIndex: Integer read GetItem write SetItem;
    property Text: String read GetText write SetText;
    property Height stored False;
    property Align;
{$IFDEF Ver120}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property MaxLength: Integer read GetMaxLength write SetMaxLength default 0;
    property ParentColor default False;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted: Boolean read GetSorted write SetSorted;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read GetOnChange write SetOnChange;
    property OnDropDown: TNotifyEvent read GetOnDropDown write SetOnDropDown;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

  TXLComboBox = class(TXLPickEdit)
  private
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
    procedure XLSaveCell(var Msg: TXLEditCell); message XL_SAVECELL;
  protected
    function AdjustHeight: Integer; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TXLDateTimePicker = class(TWinControl)
  private
    FPicker: TDateTimePicker;
    function GetDateTime: TDateTime;
    procedure SetDate(const Value: TDateTime);
    procedure SetTime(const Value: TDateTime);
  protected
    procedure AlignControls(AControl: TControl; var ARect: TRect); override;
    procedure XLEditInfo(var Msg: TXLEditInfo); message XL_EDITINFO;
    procedure XLSaveCell(var Msg: TXLEditCell); message XL_SAVECELL;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
  public
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(AOwner: TComponent); override;
    property Date: TDateTime read GetDateTime write SetDate;
    property Time: TDateTime read GetDateTime write SetTime;
  end;

  TXLDateEdit = class(TXLDateTimePicker)
  public
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
  end;

  TXLTimeEdit = class(TXLDateTimePicker)
  public
    procedure XLLoadCell(var Msg: TXLEditCell); message XL_LOADCELL;
  end;

function CalcTextHeight(AControl: TControl; AIndent: Integer; AAlign: TAlignment; const Text: String): Integer;

implementation

procedure SetEditIndent(AEdit: THandle; AIndent, ARight: Integer; AAlign: TAlignment);
var
  Margins: Integer;
begin
  if AAlign <> taLeftJustify then Inc(ARight, AIndent);
  Margins := ARight shl 16;
  if AAlign <> taRightJustify then Margins := Margins or AIndent;
  SendMessage(AEdit, EM_SETMARGINS, EC_LEFTMARGIN or EC_RIGHTMARGIN, Margins);
end;

function CalcTextHeight(AControl: TControl; AIndent: Integer; AAlign: TAlignment; const Text: String): Integer;
var
  DC: HDC;
  OldFont: THandle;
  R: TRect;
begin
  DC := GetDC(0);
  OldFont := SelectObject(DC, TEdit(AControl).Font.Handle);
  try
    R := Rect(0, 0, AControl.Width, 10240);
    if AIndent > 0 then begin
      if AAlign <> taRightJustify then Inc(R.Left, AIndent);
      if AAlign <> taLeftJustify then Dec(R.Right, AIndent);
    end;
    DrawText(DC, PChar(Text), -1, R, DT_CALCRECT or DT_WORDBREAK);
    Result := R.Bottom;
    if (AControl.Parent <> Nil) and (Result > AControl.Parent.ClientHeight) then Result := AControl.Parent.ClientHeight;
  finally
    SelectObject(DC, OldFont);
    ReleaseDC(0, DC);
  end;
end;

{ TXLMaskEdit }

procedure ToEnd(Self: TCustomEdit);
begin
  PostMessage(Self.Handle, EM_SETSEL, -1, -1);
end;

constructor TXLMaskEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BorderStyle := bsNone;
  ControlStyle := ControlStyle - [csSetCaption];
  HideSelection := True;
  AutoSize := False;
end;

const
  AlignParams: Array[TAlignment] of DWORD = (ES_LEFT, ES_RIGHT, ES_CENTER);

procedure TXLMaskEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := AlignParams[FAlign] or Params.Style or WS_CLIPCHILDREN;
end;

procedure TXLMaskEdit.XLLoadCell(var Msg: TXLEditCell);
begin
  EditMask := Msg.Cell.Format;
  Text := Msg.Cell.Text;
  UpdateHeight;
  if not IsMasked then ToEnd(Self);
end;

procedure TXLMaskEdit.XLSaveCell(var Msg: TXLEditCell);
begin
  Msg.Cell.Value := Text;
end;

procedure TXLMaskEdit.SetAlign(Value: TAlignment);
begin
  if FAlign <> Value then begin
    FAlign := Value;
    RecreateWnd;
  end;
end;

procedure TXLMaskEdit.WMSetFont(var Message: TWMSetFont);
begin
  inherited;
  UpdateParams;
end;

procedure TXLMaskEdit.CreateWnd;
begin
  inherited CreateWnd;
  UpdateParams;
end;

procedure TXLMaskEdit.XLLayout(var Msg: TXLCellLayout);
begin
  Indent := Msg.Layout^.Indent;
  Alignment := TAlignment(Msg.Layout^.HorzAlign);
end;

procedure TXLMaskEdit.CMExit(var Msg: TMessage);
begin
{We don't need any validation processing during lost focus}
  DoExit;
end;

procedure TXLMaskEdit.PostValidate;
begin
end;

procedure TXLMaskEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (SelLength <= 1) and IsMasked then begin
    if (SelStart > 0) and (Key = VK_LEFT) then exit;
    if (SelStart < Length(Text)) and (Key = VK_RIGHT) then exit;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TXLMaskEdit.CNKeyDown(var Msg: TWMKeyDown);
begin
  if Msg.CharCode = VK_DELETE then begin
     Dec(Msg.Msg, CN_BASE);
     Dispatch(Msg);
   end else inherited;
end;

procedure TXLMaskEdit.SetIndent(const Value: Integer);
begin
  if Value <> Indent then begin
    FIndent := Value;
    UpdateParams;
  end;
end;

procedure TXLMaskEdit.UpdateParams;
begin
  if HandleAllocated
  then SetEditIndent(Handle, Indent, 0, Alignment);
end;

procedure TXLMaskEdit.UpdateHeight;
begin
  Height := CalcTextHeight(Self, Indent, Alignment, Text + '.');
end;

procedure TXLMaskEdit.Change;
begin
  UpdateHeight;
  inherited Change;
end;

{ TNumEdit }

function NumericCharset(AIntMode, ANegative, AFormat: Boolean): String;
var DS: Char;
begin
  Result := '0123456789+';
  if ANegative then Result := Result + '-';
  if AFormat then DS := '.' else DS := DecimalSeparator;
  if AFormat then Result := Result + '#,';
  if not AIntMode then Result := Result + DS + 'Ee';
end;

function LeaveOnly(const ASource, AChars: String): String;
var I: Integer;
begin
  Result := ASource;
  for I := Length(Result) downto 1 do
  if Pos(Result[I], AChars) = 0 then Delete(Result, I, 1);
end;

procedure TXLNumEdit.UpdateParams;
begin
  if HandleAllocated
  then SetEditIndent(Handle, Indent, GetSystemMetrics(SM_CXVSCROLL) * Byte(Spinner), Alignment);
end;

procedure TXLNumEdit.UpdateText;
begin
  FLocked := True;
  try
    Text := FormatFloat(Format, Value);
  finally
    Flocked := False;
  end;
end;

procedure TXLNumEdit.PostValidate;
var AText: String;
begin
  if not FLocked then
  begin
    AText := LeaveOnly(Text, NumericCharset(FIntMode, True, False));
    if AText = '' then FValue := 0
    else FValue := StrToFloat(AText);
    if UseRange and ((Value < MinValue) or (Value > MaxValue))
    then InvalidOp('Value out of range!');
  end;
end;

function  TXLNumEdit.UseRange: Boolean;
begin
  Result := FValueMin < FValueMax;
end;

constructor TXLNumEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Format := '0';
end;

procedure TXLNumEdit.SetValue(const AFloat: Double);
begin
  FValue := AFloat;
  if UseRange then begin
    if AFloat > MaxValue then FValue := MaxValue else
    if AFloat < MinValue then FValue := MinValue;
  end;
  if FIntMode then FValue := Round(FValue);
  UpdateText;
end;

procedure TXLNumEdit.SetMax(const AMax: Double);
begin
  FValueMax := AMax;
  Change;
end;

procedure TXLNumEdit.SetMin(const AMin: Double);
begin
  FValueMin := AMin;
  Change;
end;

procedure TXLNumEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if not IsValidChar(Key) then Key := #0;
end;

function TXLNumEdit.IsValidChar(Key: Char): Boolean;
begin
  Result := (Pos(Key, NumericCharset(FIntMode or (Pos(DecimalSeparator, Text) <> 0), not UseRange or (MinValue < 0), False) + #8) <> 0) and (Key <> #13);
end;

procedure TXLNumEdit.SetFormat(const AFormat: String);
var
  I: Integer;
  AF: String;
begin
  AF := UnpackPrefix(UnpackPrefix(AFormat, '|'), ';');
  I := Pos('%', AF);
  AF := LeaveOnly(AF, NumericCharset(False, True, True));
  if AF <> FFormat then begin
    FIntMode := Pos('.', AF) = 0;
    if not FIntMode and (I > 0) and (AF > '') then AF := AF + '##';
    FFormat := AF;
    UpdateText;
    Increment := 1;
    if not IntMode then
      for I := Pos('.', Format) + 1 to Length(Format) do
        if Format[I] in ['0','#'] then Increment := Increment / 10;
  end;
  AF := AnsiUpperCase(UnpackSuffix(AFormat,'|'));
  Spinner := Pos('S', AF) > 0;
end;

procedure TXLNumEdit.XLLoadCell(var Msg: TXLEditCell);
begin
  Value := -1;
  Format := Msg.Cell.Format;
  try
    if Format = '' then FIntMode := (Msg.Cell is TXLCInteger);
    Value := Msg.Cell.Value;
  except
    Value := 0;
  end;
  ToEnd(Self);
end;

procedure TXLNumEdit.XLSaveCell(var Msg: TXLEditCell);
begin
  Msg.Cell.Value := Value;
end;

function TXLNumEdit.GetValue: Double;
begin
  PostValidate;
  Result := FValue;
end;

procedure TXLNumEdit.AlignControls(AControl: TControl; var ARect: TRect);
var
  BW: Integer;
begin
  if Spinner then begin
    ARect := ClientRect;
    BW := GetSystemMetrics(SM_CXVSCROLL);
    ARect.Left := ARect.Right - BW;
    BW := BW * 2;
    if ARect.Bottom > BW then ARect.Bottom := BW;
    FBar.BoundsRect := ARect;
  end;
end;

function TXLNumEdit.GetSpinner: Boolean;
begin
  Result := FBar <> Nil;
end;

procedure TXLNumEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (ssShift in Shift) then
  case Key of
    VK_UP  : begin Value := Value + Increment; Key := 0; end;
    VK_DOWN: begin Value := Value - Increment; Key := 0; end;
  end;
  if Key <> 0 then inherited KeyDown(Key, Shift);
end;

procedure TXLNumEdit.OnScroll(Sender: TObject);
begin
  if FBar.Position > 0 then Value := Value - Increment
  else if FBar.Position < 0 then Value := Value + Increment;
  FBar.Position := 0;
end;

procedure TXLNumEdit.SetSpinner(const Value: Boolean);
begin
  if Value <> Spinner then begin
    if Value then begin
      FBar := TScrollBar.Create(Self);
      FBar.Kind := sbVertical;
      FBar.Align := alRight;
      FBar.ParentCtl3D := False;
      FBar.Ctl3D := False;
      FBar.Visible := True;
      FBar.Min := -100;
      FBar.Max := 100;
      FBar.OnChange := OnScroll;
      FBar.TabStop := False;
      FBar.Parent := Self;
      Realign;
    end else begin
      FBar.Free;
      FBar := Nil;
    end;
    UpdateParams;
  end;
end;

{ TXLMemo }

constructor TXLMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BorderStyle := bsNone;
end;

procedure TXLMemo.XLLoadCell(var Msg: TXLEditCell);
begin
  if Msg.Cell.AsObject is TStrings
  then Lines.Assign(Msg.Cell.AsObject)
  else Lines.Text := Msg.Cell.Value;
  UpdateHeight;
  ToEnd(Self);
end;

procedure TXLMemo.XLSaveCell(var Msg: TXLEditCell);
begin
  Msg.Cell.AsString := Lines.Text;
end;

procedure TXLMemo.XLLayout(var Msg: TXLCellLayout);
begin
  Indent := Msg.Layout^.Indent;
  Alignment := TAlignment(Msg.Layout^.HorzAlign);
end;

procedure TXLMemo.Change;
begin
  UpdateHeight;
  inherited Change;
end;

procedure TXLMemo.UpdateHeight;
begin
  Height := CalcTextHeight(Self, Indent, Alignment, Lines.Text + '.');
end;

procedure TXLMemo.WMSetFont(var Message: TWMSetFont);
begin
  inherited;
  SetEditIndent(Handle, Indent, 0, Alignment);
end;

procedure TXLMemo.CreateWnd;
begin
  inherited CreateWnd;
  SetEditIndent(Handle, Indent, 0, Alignment);
end;

procedure TXLMemo.SetIndent(Value: Integer);
begin
  if Value <> Indent then begin
    FIndent := Value;
    if HandleAllocated then SetEditIndent(Handle, Indent, 0, Alignment);
  end;
end;

{ TPickList }

procedure TPickList.Change;
begin
  Click;
  inherited Change;
end;

constructor TPickList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csNoDesignVisible];
end;

procedure TPickList.CreateWnd;
begin
  inherited CreateWnd;
  SendParams;
end;

procedure TPickList.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Style in [csDropDown, csSimple]) and (Key <> #8)
  then PostMessage(Handle, WM_PULATTACH, 0, 0);
end;

procedure TPickList.SendParams;
var
  EWS: Cardinal;
begin
  if HandleAllocated and (EditHandle <> 0) then begin
    EWS := GetWindowLong(EditHandle, GWL_STYLE);
    EWS := EWS and not (ES_LEFT or ES_RIGHT or ES_CENTER);
    EWS := EWS or AlignParams[FAlign] or ES_MULTILINE;
    SetWindowLong(EditHandle, GWL_STYLE, EWS);
    SendMessage(EditHandle, EM_SETMARGINS, EC_LEFTMARGIN or EC_RIGHTMARGIN, FIndent);
  end;
end;

procedure TPickList.SetAlign(Value: TAlignment);
begin
  if FAlign <> Value then begin
    FAlign := Value;
    SendParams;
  end;
end;

procedure TPickList.SetIndent(Value: Integer);
begin
  if FIndent <> Value then begin
    FIndent := Value;
    SendParams;
  end;
end;

procedure TPickList.WMPULAttach(var Msg: TMessage);
var
  SelStart,
  SelEnd: Integer;
  NewText: String;
  NewItem: Integer;
begin
  SendMessage(Handle, CB_GETEDITSEL, Integer(@SelStart), Integer(@SelEnd));
  NewText := Copy(Text, 0, SelStart);
  NewItem := SendMessage(Handle, CB_FINDSTRING, 0, Integer(PChar(Text)));
  if NewItem <> CB_ERR then begin
    Text := NewText + Copy(Items[NewItem], SelStart + 1, 1024);
    SendMessage(Handle, CB_SETEDITSEL, 0, Length(Text) or (Length(NewText) shl 16));
  end;
  Change;
  Msg.Result := 0;
end;

{ TXLPickEdit }

constructor TXLPickEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csFixedHeight];
  ParentColor := False;
  FPickList := TPickList.Create(Self);
  FPickList.Visible := True;
  FPickList.Parent := Self;
  FPickList.ParentFont := True;
  FPickList.ParentColor := True;
end;

function TXLPickEdit.AdjustHeight: Integer;
begin
  Result := CalcTextHeight(Self, 0, taLeftJustify, 'Aj');
end;

procedure TXLPickEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AdjustHeight);
end;

procedure TXLPickEdit.AlignControls(AControl: TControl; var ARect: TRect);
var
  AW: Integer;
  R: TRect;
begin
  with FPickList do begin
    AW := 0;
    if (Style = csDropDown) then begin
      if (EditHandle <> 0) then begin
        Windows.GetWindowRect(EditHandle, R);
        AW := R.Right;
        Windows.GetWindowRect(Handle, R);
        AW := R.Right - AW;
      end else AW := 15;
    end;
    Inc(AW, Self.ClientWidth);
    BoundsRect := Rect(-3, -3, AW + 3, Self.ClientHeight + 3);
  end;
  inherited AlignControls(AControl, ARect);
end;

procedure TXLPickEdit.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  Height := 1024;
end;

function TXLPickEdit.GetDroppedDown: Boolean;
begin
  Result := FPickList.DroppedDown;
end;

function TXLPickEdit.GetItem: Integer;
begin
  Result := FPickList.ItemIndex;
  if Result = -1
  then Result := FPickList.Perform(CB_FINDSTRINGEXACT, 0, Integer(PChar(Text)));
end;

function TXLPickEdit.GetItems: TStrings;
begin
  Result := FPickList.Items;
end;

function TXLPickEdit.GetMaxLength: Integer;
begin
  Result := FPickList.MaxLength;
end;

function TXLPickEdit.GetOnChange: TNotifyEvent;
begin
  Result := FPickList.OnChange;
end;

function TXLPickEdit.GetOnDropDown: TNotifyEvent;
begin
  Result := FPickList.OnDropDown;
end;

function TXLPickEdit.GetSorted: Boolean;
begin
  Result := FPickList.Sorted;
end;

function TXLPickEdit.GetText: String;
begin
  Result := FPickList.Text;
end;

procedure TXLPickEdit.SetDroppedDown(Value: Boolean);
begin
  FPickList.DroppedDown := Value;
end;

procedure TXLPickEdit.SetItem(Value: Integer);
begin
  FPickList.ItemIndex := Value;
end;

procedure TXLPickEdit.SetItems(Value: TStrings);
begin
  FPickList.Items.Assign(Value);
end;

procedure TXLPickEdit.SetMaxLength(Value: Integer);
begin
  FPickList.MaxLength := Value;
end;

procedure TXLPickEdit.SetOnChange(Value: TNotifyEvent);
begin
 FPickList.OnChange := Value;
end;

procedure TXLPickEdit.SetOnDropDown(Value: TNotifyEvent);
begin
 FPickList.OnDropDown := Value;
end;

procedure TXLPickEdit.SetSorted(Value: Boolean);
begin
  FPickList.Sorted := Value;
end;

procedure TXLPickEdit.SetText(const Value: String);
begin
  FPickList.Text := Value;
end;

type
  PXLAddToList = ^TXLAddToList;
  TXLAddToList = record
    WorkList,
    RealList: TStringList;
  end;

procedure TXLPickEdit.AddToList(ACell: TXLCell; AData: Integer; var Stop: Boolean);
begin
//  if ACell.Width = 1 then
  with PXLAddToList(AData)^ do begin
    try
      WorkList.Text := ACell.AsString;
      if WorkList.Count > 0 then RealList.Add(WorkList[0]);
      Stop := RealList.Count = 50;
    except
    end;
  end;
end;

procedure TXLPickEdit.XLLoadCell(var Msg: TXLEditCell);
var
  R: TRect;
  ATL: TXLAddToList;
begin
  if Msg.Cell.AsObject is TStrings then begin
    Items := TStrings(Msg.Cell.AsObject);
    ItemIndex := Msg.Cell.AsInteger;
  end else with Msg.Cell do begin
    Items.Clear;
    ATL.WorkList := TStringList.Create;
    ATL.RealList := TStringList.Create;
    ATL.RealList.Sorted := True;
    ATL.RealList.Duplicates := dupIgnore;
    try
      R := Bounds;
      R.Top := 0;
      R.Right := R.Left + 1;
      Sheet.EnumCells(R, False, AddToList, Integer(@ATL));
    finally
      Items := ATL.RealList;
      ATL.RealList.Free;
      ATL.WorkList.Free;
    end;
    Self.Text := Text;
  end;
end;

procedure TXLPickEdit.XLSaveCell(var Msg: TXLEditCell);
begin
  if Msg.Cell.AsObject is TStrings
  then Msg.Cell.AsInteger := ItemIndex
  else Msg.Cell.Value := Text;
end;

procedure TXLPickEdit.XLLayout(var Msg: TXLCellLayout);
begin
  FPickList.Indent := Msg.Layout^.Indent;
  FPickList.Alignment := TAlignment(Msg.Layout^.HorzAlign);
end;

procedure TXLPickEdit.XLEditInfo(var Msg: TXLEditInfo);
begin
  Msg.Control^ := FPickList;
  Msg.Handle^ := FPickList.EditHandle;
end;

{ TXLComboBox }

constructor TXLComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPickList.Style := csOwnerDrawFixed;
end;

procedure TXLComboBox.XLLoadCell(var Msg: TXLEditCell);
begin
  if Msg.Cell.AsObject is TStrings then begin
    FPickList.Items := TStrings(Msg.Cell.AsObject);
    FPickList.ItemIndex := Msg.Cell.AsInteger;
  end else Items.Text := Msg.Cell.Value;
end;

procedure TXLComboBox.XLSaveCell(var Msg: TXLEditCell);
begin
  Msg.Cell.AsInteger := ItemIndex;
end;

function TXLComboBox.AdjustHeight: Integer;
begin
  Result := inherited AdjustHeight;
  FPickList.ItemHeight := Result;
  FPickList.RecreateWnd;
end;

{ TXLDate }

procedure TXLDateEdit.XLLoadCell(var Msg: TXLEditCell);
begin
  Date := Msg.Cell.Value;
end;

{ TXLTimeEdit }

procedure TXLTimeEdit.XLLoadCell(var Msg: TXLEditCell);
begin
  Time := Msg.Cell.Value;
end;

{ TXLDateTimePicker }

constructor TXLDateTimePicker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPicker := TDateTimePicker.Create(Self);
  FPicker.ParentColor := True;
  FPicker.ParentFont := True;
  FPicker.Visible := True;
  FPicker.DateMode := dmUpDown;
  FPicker.Parent := Self;
  TabStop := False;
end;

procedure TXLDateTimePicker.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  Height := 1024;
end;

procedure TXLDateTimePicker.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, CalcTextHeight(Self, 0, taLeftJustify, 'Aj'));
end;

procedure TXLDateTimePicker.AlignControls(AControl: TControl; var ARect: TRect);
begin
  InflateRect(ARect, 3, 3);
  FPicker.BoundsRect := ARect;
end;

function TXLDateTimePicker.GetDateTime: TDateTime;
begin
  Result := FPicker.Date;
end;

procedure TXLDateTimePicker.SetDate(const Value: TDateTime);
begin
  FPicker.Kind := dtkDate;
  FPicker.Date := Value;
end;

procedure TXLDateTimePicker.SetTime(const Value: TDateTime);
begin
  FPicker.Kind := dtkTime;
  FPicker.Time := Value;
end;

procedure TXLDateTimePicker.XLSaveCell(var Msg: TXLEditCell);
begin
  Msg.Cell.Value := GetDateTime;
end;

procedure TXLDateTimePicker.XLEditInfo(var Msg: TXLEditInfo);
begin
  Msg.Control^ := FPicker;
  Msg.Handle^ := FPicker.Handle;
end;

initialization
  with XLManager do begin
    RegisterClasses([TXLCString], [TXLPickEdit, TXLMemo, TXLMaskEdit]);
    RegisterClasses([TXLCStrings], [TXLMemo, TXLPickEdit, TXLComboBox]);
    RegisterClasses([TXLCInteger, TXLCDouble, TXLCCurrency], [TXLNumEdit]);
    RegisterClasses([TXLCDateTime], [TXLPickEdit]);
    RegisterClasses([TXLCDateTime], [TXLDateEdit, TXLTimeEdit]);
  end;
end.
