unit XLFonts;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, XLEdits;

type
  TProPickBox = class(TPickList)
  private
    FDynamic: Boolean;
    function GetIndex: Integer;
    procedure SetIndex(Value: Integer);
  protected
    property DynamicItems: Boolean read FDynamic write FDynamic;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure CreateList; dynamic;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
    property ItemIndex: Integer read GetIndex write SetIndex;
    property Items;
  published
    property Style;
{$IFDEF Ver120}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
{$IFDEF Ver120}
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

  TPickBox = class(TProPickBox)
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Items;
  end;

  TFontBox = class;

  TFontOutlineBox = class(TProPickBox)
  private
    FBox: TFontBox;
    FStyles: TFontStyles;
    procedure SetFontStyles(AStyles: TFontStyles);
  protected
    procedure CreateList; override;
    procedure Click; override;
  published
    property  FontStyles: TFontStyles read FStyles write SetFontStyles;
  end;

  TFontSizeBox = class(TProPickBox)
  private
    FBox: TFontBox;
    FFontSize: Integer;
    procedure SetSize(Value: Integer);
  protected
    procedure KeyPress(var Key: Char); override;
    procedure Click; override;
    procedure CreateList; override;
  published
    property  FontSize: Integer read FFontSize write SetSize;
  end;

  TFontBox = class(TProPickBox)
  private
    FOutline: TFontOutlineBox;
    FSize: TFontSizeBox;
    FFont: TFont;
    FFontChanged: TNotifyEvent;
    FTTOnly: Boolean;
    procedure SetFont(Value: TFont);
    procedure SetOutline(Value: TFontOutlineBox);
    procedure SetSize(Value: TFontSizeBox);
    procedure BuildOutline;
    procedure BuildSize;
    procedure BuildName;
    procedure FontChanged(Sender: TObject);
    procedure SetTTOnly(Value: Boolean);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
    procedure CreateList; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property  SelFont: TFont read FFont write SetFont;
    property  TTOnly: Boolean read FTTOnly write SetTTOnly default False;
    property  OutlineBox: TFontOutlineBox read FOutline write SetOutline;
    property  SizeBox: TFontSizeBox read FSize write SetSize;
    property  OnFontChange: TNotifyEvent read FFontChanged write FFontChanged;
  end;
procedure Register;
implementation

var
  TrueTypeBMP: TBitmap;
  Fonts : TStringList;

procedure CreateBitmaps;
begin
  TrueTypeBMP := TBitmap.Create;
  TrueTypeBMP.LoadFromResourceName(HInstance, 'TRUETYPE');
  TrueTypeBMP.Transparent := True;
end;

function EnumFontFamProc(lpelf: PENUMLOGFONT; lpntm: PNEWTEXTMETRIC; FontType: Integer; AData: Integer): integer; stdcall;
var FO: Integer;
    OD: Integer;
begin
  Result := 0;
  if (FontType and RASTER_FONTTYPE) <> 0
  then Fonts.Objects[AData] := Pointer($F)
  else if (FontType and TRUETYPE_FONTTYPE) <> 0 then
  begin
    OD := Integer(Fonts.Objects[AData]) or $10000;
    if lpelf^.elfLogFont.lfWeight > 500 then FO := 1 else FO := 0;
    if lpelf^.elfLogFont.lfItalic <> 0 then FO := FO or $2;
    OD := OD or ($1 shl FO);
    Fonts.Objects[AData] := Pointer(OD);
    Result := 1;
  end;
end;

procedure CreateFonts;
var DC: HDC;
    I, J: Integer;
begin
  Fonts := TStringList.Create;
  Fonts.Duplicates := dupIgnore;
  Fonts.Sorted := True;
  DC := GetDC(0);
  try
    for I := 0 to Screen.Fonts.Count - 1 do begin
      J := Fonts.Add(Screen.Fonts[I]);
      EnumFontFamilies(DC, PChar(Fonts[J]), @EnumFontFamProc, J);
    end;
  finally
    ReleaseDC(0, DC);
  end;
  Fonts.Delete(Fonts.IndexOf('Default'));
end;

{$R *.RES}

{TProPickBox}

procedure TProPickBox.CreateWnd;
begin
  inherited;
  if FDynamic then CreateList;
  Perform(CB_SETEXTENDEDUI, 1, 0);
end;

procedure TProPickBox.DestroyWnd;
begin
  if FDynamic then Items.Clear;
  inherited;
end;

procedure TProPickBox.CreateList;
begin
end;

procedure TProPickBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := (Params.Style and not CBS_OWNERDRAWVARIABLE) or CBS_OWNERDRAWFIXED;
end;

constructor TProPickBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DynamicItems := True;
end;

function TProPickBox.GetIndex: Integer;
begin
  Result := Items.IndexOf(Text);
end;

procedure TProPickBox.SetIndex(Value: Integer);
begin
  inherited ItemIndex := Value;
end;

{ TPickBox }

constructor TPickBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DynamicItems := False;
end;

{TFontBox}

procedure TFontBox.BuildName;
begin
  Text := FFont.Name;
end;

procedure TFontBox.BuildOutline;
begin
  if FOutline <> Nil then FOutline.FontStyles := SelFont.Style;
end;

procedure TFontBox.BuildSize;
begin
  if FSize <> Nil then FSize.FontSize := SelFont.Size;
end;

procedure TFontBox.Click;
begin
  SelFont.Name := Text;
  inherited;
end;

constructor TFontBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFont := TFont.Create;
  FFont.OnChange := FontChanged;
end;

procedure TFontBox.CreateList;
var I: Integer;
begin
  Items := Fonts;
  if TTOnly then begin
    Items.BeginUpdate;
    for I := Items.Count - 1 downto 0 do
    if (Integer(Items.Objects[I]) and $10000) = 0
    then Items.Delete(I);
    Items.EndUpdate;
  end;
end;

destructor TFontBox.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;

procedure TFontBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
begin
{$IFDEF Ver120}
  TControlCanvas(Canvas).UpdateTextFlags;
{$ENDIF}
  Canvas.TextRect(Rect, Rect.Left + 20, Rect.Top, Items[Index]);
  if (Integer(Items.Objects[Index]) and $10000) <> 0
  then Canvas.Draw(Rect.Left, Rect.Top, TrueTypeBMP);
end;

procedure TFontBox.FontChanged(Sender: TObject);
begin
  BuildName;
  BuildOutline;
  BuildSize;
  if Assigned(FFontChanged) then FFontChanged(Self);
end;

procedure TFontBox.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then begin
    if AComponent = FOutline then OutlineBox := Nil else
    if AComponent = FSize then SizeBox := Nil;
  end;
end;

procedure TFontBox.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TFontBox.SetOutline(Value: TFontOutlineBox);
begin
  if FOutline <> Value then begin
    if FOutline <> Nil then FOutline.FBox := Nil;
    FOutline := Value;
    if FOutline <> Nil then FOutline.FBox := Self;
    BuildOutline;
  end;
end;

procedure TFontBox.SetSize(Value: TFontSizeBox);
begin
  if FSize <> Value then begin
    if FSize <> Nil then FSize.FBox := Nil;
    FSize := Value;
    if FSize <> Nil then FSize.FBox := Self;
    BuildSize;
  end;
end;

procedure TFontBox.SetTTOnly(Value: Boolean);
begin
  if Value <> TTOnly then begin
    FTTOnly := Value;
    RecreateWnd;
  end;
end;

{ TFontSizeBox }

procedure TFontSizeBox.Click;
begin
  if (Text > '') then FontSize := StrToInt(Text);
  inherited;
end;

procedure TFontSizeBox.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if not (Key in [#8, '0'..'9']) then Key := #0;
end;

procedure TFontSizeBox.SetSize(Value: Integer);
begin
  if Value < 1 then Value := 1;
  if FFontSize <> Value then begin
    FFontSize := Value;
    if FBox <> Nil then FBox.SelFont.Size := FFontSize;
    Text := IntToStr(FFontSize);
  end;
end;

procedure TFontSizeBox.CreateList;
begin
  Items.Text := '6'^M'7'^M'8'^M'10'^M'12'^M'14'^M'16'^M'20'^M'24'^M'32'^M'48'^M'72';
end;

{ TFontOutlineBox }
procedure TFontOutlineBox.Click;
begin
  FontStyles := (FontStyles - [fsBold, fsItalic]) + TFontStyles(Byte(ItemIndex));
  inherited;
end;

procedure TFontOutlineBox.CreateList;
begin
  Items.Text := 'Regular'^M'Bold'^M'Italic'^M'Bold Italic'^M;
  ItemIndex := 0;
end;

procedure TFontOutlineBox.SetFontStyles(AStyles: TFontStyles);
begin
  if FStyles <> AStyles then
  begin
    FStyles := AStyles;
    Text := Items[Byte(FStyles * [fsBold, fsItalic])];
    if FBox <> Nil then FBox.SelFont.Style := FStyles;
  end;
end;
procedure Register;
begin
  RegisterComponents('XLGrids',[TPickBox, TFontBox, TFontOutlineBox, TFontSizeBox]);
end;
initialization
  CreateBitmaps;
  CreateFonts;

finalization
  TrueTypeBMP.Free;
  Fonts.Free;
end.
