unit XLStyler;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  XLCell, StdCtrls, ExtCtrls, Buttons, ComCtrls, XLGrids, CheckLst,
  XLFonts, XLColors, XLEdits;

type
  TXLStylerDesigner = class(TXLCellDesigner)
    PageStyle: TTabSheet;
    GroupIncludes: TGroupBox;
    CBProps: TCheckListBox;
    BDelete: TButton;
    BAdd: TButton;
    Styles: TPickBox;
    LabelStyleName: TLabel;
    procedure StylesChange(Sender: TObject);
    procedure CBPropsClickCheck(Sender: TObject);
    procedure BAddClick(Sender: TObject);
    procedure BDeleteClick(Sender: TObject);
    procedure StylesKeyPress(Sender: TObject; var Key: Char);
  private
    FStyler: TXLStyler;
    procedure SetStyler(AStyler: TXLStyler);
    procedure LoadList;
  protected
    procedure SetChanges(AChanges: TXLProps); override;  
  public
    property Styler: TXLStyler read FStyler write SetStyler;
  end;

  function OpenXLStylerDesigner(AStyler: TXLStyler): Boolean;

implementation

{$R *.DFM}

  function OpenXLStylerDesigner(AStyler: TXLStyler): Boolean;
  var Designer: TXLStylerDesigner;
  begin
    Designer := TXLStylerDesigner.Create(nil);
    try
      Designer.Styler := AStyler;
      Result := Designer.ShowModal = mrOk;
    finally
      Designer.Free;
    end;
  end;

{ TXLStylerDesigner }

procedure TXLStylerDesigner.LoadList;
var I: Integer;
begin
  with Styles.Items do begin
    BeginUpdate;
    try
      Clear;
      if (Styler <> Nil) and (Styler.StyleCount > 0) then
      begin
        for I := 0 to Styler.StyleCount - 1 do AddObject(Styler.Styles[I].Alias, Styler.Styles[I]);
        Styles.ItemIndex := 0;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TXLStylerDesigner.SetStyler(AStyler: TXLStyler);
begin
  if AStyler <> FStyler then begin
    FStyler := AStyler;
    LoadList;
  end;
end;

procedure TXLStylerDesigner.StylesChange(Sender: TObject);
begin
  BAdd.Enabled := (Styles.ItemIndex < 0) and (Styles.Text > '');
  BDelete.Enabled := (Styles.ItemIndex >= 0);
  if BDelete.Enabled
  then Source := TXLPersistent(Styles.Items.Objects[Styles.ItemIndex])
  else Source := Nil;
  GroupIncludes.Visible := BDelete.Enabled;
end;

procedure TXLStylerDesigner.CBPropsClickCheck(Sender: TObject);
begin
  with CBProps do
  if Checked[ItemIndex] then Changes := Changes + [TXLProp(ItemIndex + 1)]
  else begin
    Changes := Changes - [TXLProp(ItemIndex + 1)];
    Source.Props := Source.Props - [TXLProp(ItemIndex + 1)];
  end;
  BApply.Enabled := True;
end;

procedure TXLStylerDesigner.BAddClick(Sender: TObject);
begin
  Styles.ItemIndex := Styles.Items.AddObject(Styles.Text, Styler.Add(Styles.Text));
  StylesChange(Sender);
end;

procedure TXLStylerDesigner.BDeleteClick(Sender: TObject);
var DS: TXLStyle;
begin
  DS := TXLStyle(Source);
  if (DS <> Nil) and (Application.MessageBox(PChar('Are you sure to delete style named ' + DS.Alias + '?'), 'XLStyler Designer', MB_OKCancel or MB_IconQuestion) = id_Ok) then begin
    Source := Nil;
    Styles.Items.Delete(Styles.ItemIndex);
    DS.Free;
    StylesChange(Self);
  end;
end;

procedure TXLStylerDesigner.SetChanges(AChanges: TXLProps);
var I: TXLProp;
begin
  inherited SetChanges(AChanges);
  if Source <> Nil then AChanges := AChanges + Source.Props;
  for I := prControl to prStyles do
  CBProps.Checked[Integer(I) - 1] := I in AChanges;
end;

procedure TXLStylerDesigner.StylesKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) and BAdd.Enabled then BAddClick(Sender);
end;

end.
 