{-----------------------------------------------------------------------------
 Unit Name: EventSink
 
 Author:    Vadim Krochak
 Copyright:  2003 All rights reserved.
 E.Mail:    vkrochak@bezeqnt.net
 License:   Don't redistribute for cash.  Don't change and
  redistribute under your name.  Let me know if you fix anything or improve
  anything.

 Purpose:   Generic COM event sink implementation.
 History:
-----------------------------------------------------------------------------}

unit EventSink;

interface

uses ComObj;

type

  TEventSink = class(TComObject, IDispatch)
  private

    FSinkedInterfaceDIID: TGUID;
    FConnection: Integer;

  protected

    { IDispatch }
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;

    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
    	Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; virtual; stdcall;

	public

  	constructor Create(const SinkedInterface: TGUID);
		function ObjQueryInterface(const IID: TGUID; out Obj): HResult; override;

    function Connect(const Source: IUnknown): Boolean;
    function Disconnect(const Source: IUnknown): Boolean;

  end;

//******************************************************************************
//******************************************************************************

implementation

uses Windows, ActiveX;

//******************************************************************************
//******************************************************************************

constructor TEventSink.Create(const SinkedInterface: TGUID);
begin
	Self.FSinkedInterfaceDIID := SinkedInterface;
end;

//******************************************************************************
function TEventSink.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
 	Result := E_NOTIMPL;
end;

//******************************************************************************
function TEventSink.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HResult;
begin
 	Result := E_NOTIMPL;
end;

//******************************************************************************
function TEventSink.GetTypeInfoCount(out Count: Integer): HResult;
begin
 	Result := E_NOTIMPL;
end;

//******************************************************************************
function TEventSink.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin
	Result := DISP_E_MEMBERNOTFOUND;
end;


{{*** VK ***********************************************************************
  Function   :ObjQueryInterface
  Purpose    :TComObject's ObjQueryInterface method override
  Parameters :
  Returns    :
*******************************************************************************}
function TEventSink.ObjQueryInterface(const IID: TGUID; out Obj): HResult;
begin

	if IsEqualGUID( Self.FSinkedInterfaceDIID, IID ) then
  begin
    if Self.GetInterface( IDispatch, Obj ) then
    begin
  	  Result := S_OK;
    end
    else
    begin
  	  Result := E_NOINTERFACE;
    end; // if-else
  end
  else
  begin
  	Result := E_NOINTERFACE;
  end;// if-else

end;

{{*** VK ***********************************************************************
  Function   :Connect
  Purpose    :connects event sink
  Parameters :Source - source interface
  Returns    :true on success
*******************************************************************************}
function TEventSink.Connect(const Source: IUnknown): Boolean;
begin

	Self.Disconnect(Source);
	InterfaceConnect( Source, Self.FSinkedInterfaceDIID, Self, Self.FConnection );
  Result := Self.FConnection <> 0;

end;

{{*** VK ***********************************************************************
  Function   :Disconnect
  Purpose    :disconnects event sink
  Parameters :Source - source interface
  Returns    :true on success
*******************************************************************************}
function TEventSink.Disconnect(const Source: IUnknown): Boolean;
begin

	if Self.FConnection <> 0 then
  begin
		InterfaceDisconnect( Source, Self.FSinkedInterfaceDIID, Self.FConnection );
  end;// if

  Result := Self.FConnection = 0;

end;

end.
