{-----------------------------------------------------------------------------
 Unit Name: ExcelAddIn_IMPL

 Author:    Vadim Krochak
 Copyright:  2003 All rights reserved.
 E.Mail:    vkrochak@bezeqnt.net
 License:   Don't redistribute for cash.  Don't change and
  redistribute under your name.  Let me know if you fix anything or improve
  anything.

 Purpose:TExcelAddIn (Excel addin) implmentation.
 History:
-----------------------------------------------------------------------------}


unit ExcelAddIn_IMPL;

interface

uses
  ComObj, ActiveX, MSOAddIn_TLB, MSOfficeAddIn, Office_TLB, MSOEventSink, StdVcl;

type

  TExcelAddIn = class(TMSOfficeAddIn, IExcelAddIn)
  private
    FCommandBar: CommandBar;

    FCommandBarBtn: CommandBarButton;
    FBtnEventSink: TCommandBarButtonEventsSink;

    function IsCmdBarExists(): Boolean;
    procedure OnBtnClick(const Ctrl: CommandBarButton; var CancelDefault: WordBool);
  
  protected
  
    procedure DoOnConnection(ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
    	var custom: PSafeArray); override;
    procedure DoOnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); override;
    procedure DoOnAddInsUpdate(var custom: PSafeArray); override;
    procedure DoOnStartupComplete(var custom: PSafeArray); override;
    procedure DoOnBeginShutdown(var custom: PSafeArray); override;

    function BindToHost(): Boolean;
    function UnbindFromHost(): Boolean;

  end;

implementation

uses Windows, ComServ, SysUtils, Controls, Excel_TLB, MSOAddIn_Defs, MSOAddIn_Utils, Form_AddrInfo_U;


//******************************************************************************
//******************************************************************************

procedure TExcelAddIn.DoOnAddInsUpdate(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TExcelAddIn.DoOnAddInsUpdate'#10);
  {$ENDIF}
end;

//******************************************************************************
procedure TExcelAddIn.DoOnBeginShutdown(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TExcelAddIn.DoOnBeginShutdown'#10);
  {$ENDIF}
end;

//******************************************************************************
procedure TExcelAddIn.DoOnConnection(ConnectMode: ext_ConnectMode;
  const AddInInst: IDispatch; var custom: PSafeArray);
begin
  Self.BindToHost();
end;

//******************************************************************************
procedure TExcelAddIn.DoOnDisconnection(RemoveMode: ext_DisconnectMode;
  var custom: PSafeArray);
begin
  Self.UnbindFromHost();

  // removed by user from COM add-in manager:
  if RemoveMode = ext_dm_UserClosed then
  begin
    Self.FCommandBar.Delete();        
  end; // if
  
end;

//******************************************************************************
procedure TExcelAddIn.DoOnStartupComplete(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TExcelAddIn.DoOnStartupComplete'#10);
  {$ENDIF}
end;

{{*** VK ***********************************************************************
  Function   :IsCmdBarExists
  Purpose    :checks if addin's toolbar exists
  Parameters :
  Returns    :true if exists
*******************************************************************************}
function TExcelAddIn.IsCmdBarExists: Boolean;
var
	BarIndex: Integer;
begin

  BarIndex := MSOAddIn_Utils.FindCommandBar( Self.FCommandBars, EXCELADDIN_CMD_BAR_NAME );
  Result := BarIndex > 0;
  if Result then
  begin
    Self.FCommandBar := Self.FCommandBars.Item[BarIndex];
  end; // if

end;

{{*** VK ***********************************************************************
  Function   :OnBtnClick
  Purpose    :addin's command button OnClick event handler
  Parameters :
  Returns    :
*******************************************************************************}
procedure TExcelAddIn.OnBtnClick(const Ctrl: CommandBarButton; var CancelDefault: WordBool);
var
	Form_AddrInfo: TForm_AddrInfo;
begin

	Form_AddrInfo := TForm_AddrInfo.Create(nil);
  if Form_AddrInfo.ShowModal = mrOK then
  begin

  	try

      with Self.FHostApp as ExcelApplication do
      begin

        if Workbooks.Count = 0 then
        begin
          Workbooks.Add( EmptyParam, LOCALE_SYSTEM_DEFAULT );
        end; // if

        with ActiveWorkbook.ActiveSheet as ExcelWorksheet do
        begin
          Cells.Item[ActiveCell.Row, ActiveCell.Column] := 'Name:';
          Cells.Item[ActiveCell.Row, ActiveCell.Column + 1] := Form_AddrInfo.Edit_Name.Text ;

          Cells.Item[ActiveCell.Row + 1, ActiveCell.Column] := 'Phone:';
          Cells.Item[ActiveCell.Row + 1, ActiveCell.Column + 1] := Form_AddrInfo.Edit_Phone.Text ;

          Cells.Item[ActiveCell.Row + 2, ActiveCell.Column] := 'E-Mail:';
          Cells.Item[ActiveCell.Row + 2, ActiveCell.Column + 1] := Form_AddrInfo.Edit_Email.Text ;
        end; // with

      end; // with

    except
      on E: Exception do
      begin
        {$IFDEF DEBUG}
        OutputDebugString(PChar(E.Message));
        {$ENDIF}
      end;
    end;// try-except

  end;// if

	Form_AddrInfo.Free;

end;

{{*** VK ***********************************************************************
  Function   :BindToHost
  Purpose    :binds addin to host's GUI
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TExcelAddIn.BindToHost: Boolean;
begin

	try

    if Self.IsCmdBarExists() then
    begin
      //(VK): find existing button:
      FCommandBarBtn := Self.FCommandBar.FindControl( msoControlButton, 1, EXCELADDIN_BTN_NAME + TAG_SUFFIX, 1, 1 )  as CommandBarButton;
    end
    else
    begin

    	//(VK): add command bar:
      Self.FCommandBar := Self.FCommandBars.Add( EXCELADDIN_CMD_BAR_NAME, msoBarTop, 0, False );

			//(VK): add button:
      Self.FCommandBarBtn := Self.FCommandBar.Controls.Add( msoControlButton, 1, 0, 1, False ) as CommandBarButton;

      Self.FCommandBarBtn.Set_Tag( EXCELADDIN_BTN_NAME + TAG_SUFFIX );
      Self.FCommandBarBtn.Set_TooltipText( EXCELADDIN_BTN_NAME );
      Self.FCommandBarBtn.Set_Style( msoButtonCaption );
      Self.FCommandBarBtn.Set_Caption( EXCELADDIN_BTN_NAME );

      // user should see newly created command bar:
      Self.FCommandBar.Set_Visible( True );

    end;// if-else

  except
    on E: Exception do
    begin
			{$IFDEF DEBUG}
      OutputDebugString(Pchar(E.Message));
      {$ENDIF}
			Result := False; Exit;
    end;
  end;// try-except


  //(VK): 'connect' button to event sink:
  if Self.FBtnEventSink = nil then
  	Self.FBtnEventSink := TCommandBarButtonEventsSink.Create(DIID__CommandBarButtonEvents);

	Self.FBtnEventSink.OnCommandBarButtonClick := Self.OnBtnClick;

	Result := Self.FBtnEventSink.Connect( Self.FCommandBarBtn );

end;

{{*** VK ***********************************************************************
  Function   :UnbindFromHost
  Purpose    :unbinds addin from host's GUI
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TExcelAddIn.UnbindFromHost: Boolean;
begin
 	Result := Self.FBtnEventSink.Disconnect( Self.FCommandBarBtn );
  Self.FBtnEventSink := nil;
end;

//******************************************************************************
//******************************************************************************

initialization

  TMSOAddInFactory.Create( EXCEL_APP, EXCELADDIN_NAME,
    EXCELADDIN_DESCRIPTION, $03,
    ComServer, TExcelAddIn, Class_ExcelAddIn,
    ciMultiInstance, tmApartment
  );

end.
