{-----------------------------------------------------------------------------
 Unit Name: MSOEventSink

 Author:    Vadim Krochak
 Copyright:  2003 All rights reserved.
 E.Mail:    vkrochak@bezeqnt.net
 License:   Don't redistribute for cash.  Don't change and
  redistribute under your name.  Let me know if you fix anything or improve
  anything.


 Purpose: TCommandBarButtonEventsSink, TCommandBarsEventsSink and
          TCommandBarComboBoxEventsSink implementation
 History:
-----------------------------------------------------------------------------}

unit MSOEventSink;

interface

uses EventSink, Office_TLB;

type

  TOnCommandBarButtonClick = procedure(const Ctrl: CommandBarButton; var CancelDefault: WordBool) of object;

  TCommandBarButtonEventsSink = class(TEventSink)
  private
  	FOnCommandBarButtonClick: TOnCommandBarButtonClick;

	protected
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; override;

		procedure ProcessEvent(Params: Pointer);

  public
  	property OnCommandBarButtonClick: TOnCommandBarButtonClick read FOnCommandBarButtonClick write FOnCommandBarButtonClick;

  end;


	TOnCommandBarsUpdate = procedure() of object;

  TCommandBarsEventsSink = class(TEventSink)
  private
  	FOnCommandBarsUpdate: TOnCommandBarsUpdate;
  protected
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; override;

		procedure ProcessEvent(Params: Pointer);
  public
  	property OnCommandBarsUpdate: TOnCommandBarsUpdate read FOnCommandBarsUpdate write FOnCommandBarsUpdate;
  end;

  TOnCommandBarComboBoxChange = procedure(const Ctrl: CommandBarComboBox) of object;

  TCommandBarComboBoxEventsSink = class(TEventSink)
  private
  	FOnCommandBarComboBoxChange: TOnCommandBarComboBoxChange;
  protected
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; override;

		procedure ProcessEvent(Params: Pointer);
  public
  	property OnCommandBarComboBoxChange: TOnCommandBarComboBoxChange read FOnCommandBarComboBoxChange write FOnCommandBarComboBoxChange;
  end;

//******************************************************************************
//******************************************************************************

implementation

uses Windows, ActiveX;

//******************************************************************************
//******************************************************************************

function TCommandBarButtonEventsSink.Invoke(DispID: Integer;
  const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult,
  ExcepInfo, ArgErr: Pointer): HResult;
begin

	if ( DispID = 1 ) and ( TDispParams(Params).cArgs = 2 ) then
  begin
		Self.ProcessEvent(Pointer(@Params));
    Result := S_OK;
  end
  else
  begin
  	Result := inherited Invoke(DispID, IID, LocaleID, Flags, Params, VarResult, ExcepInfo, ArgErr );
  end;// if-else

end;

//******************************************************************************
procedure TCommandBarButtonEventsSink.ProcessEvent(Params: Pointer);
var
  CanCancel: WordBool;
begin

  if Assigned(Self.FOnCommandBarButtonClick) then
  begin

    CanCancel := PDispParams(Params)^.rgvarg[0].pbool^;
  	Self.FOnCommandBarButtonClick(
    	CommandBarButton(PDispParams(Params)^.rgvarg[1].pdispVal), CanCancel
    );

    PDispParams(Params)^.rgvarg[0].pbool^ := CanCancel;

  end;// if

end;

{ TCommandBarsEventsSink }

//******************************************************************************
procedure TCommandBarsEventsSink.ProcessEvent(Params: Pointer);
begin

  if Assigned(Self.FOnCommandBarsUpdate) then
  begin
  	Self.FOnCommandBarsUpdate();
  end;// if

end;

//******************************************************************************
function TCommandBarsEventsSink.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin

	if DispID = 1 then
  begin
		Self.ProcessEvent(Pointer(@Params));
    Result := S_OK;
  end
  else
  begin
  	Result := inherited Invoke(DispID, IID, LocaleID, Flags, Params, VarResult, ExcepInfo, ArgErr );
  end;// if-else

end;


{ TCommandBarComboBoxEventsSink }

//******************************************************************************
procedure TCommandBarComboBoxEventsSink.ProcessEvent(Params: Pointer);
begin

  if Assigned(Self.FOnCommandBarComboBoxChange) then
  begin

  	Self.FOnCommandBarComboBoxChange(
    	CommandBarComboBox( PDispParams(Params)^.rgvarg[0].pdispVal )
    );

  end;// if

end;

//******************************************************************************
function TCommandBarComboBoxEventsSink.Invoke(DispID: Integer;
  const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult,
  ExcepInfo, ArgErr: Pointer): HResult;
begin

	if ( DispID = 1 ) and ( TDispParams(Params).cArgs = 1 ) then
  begin
		Self.ProcessEvent(Pointer(@Params));
    Result := S_OK;
  end
  else
  begin
  	Result := inherited Invoke(DispID, IID, LocaleID, Flags, Params, VarResult, ExcepInfo, ArgErr );
  end;// if-else

end;

end.
