{-----------------------------------------------------------------------------
 Unit Name: MSOfficeAddIn

 Author:    Vadim Krochak
 Copyright:  2003 All rights reserved.
 E.Mail:    vkrochak@bezeqnt.net
 License:   Don't redistribute for cash.  Don't change and
  redistribute under your name.  Let me know if you fix anything or improve
  anything.

 Purpose: TMSOfficeAddIn and TMSOAddInFactory implementation
 History:
-----------------------------------------------------------------------------}

unit MSOfficeAddIn;

interface

uses
  ComObj, ActiveX, Office_TLB;

// Constants for enum ext_ConnectMode
type
  ext_ConnectMode = TOleEnum;
const
  ext_cm_AfterStartup = $00000000;
  ext_cm_Startup = $00000001;
  ext_cm_External = $00000002;
  ext_cm_CommandLine = $00000003;

// Constants for enum ext_DisconnectMode
type
  ext_DisconnectMode = TOleEnum;
const
  ext_dm_HostShutdown = $00000000;
  ext_dm_UserClosed = $00000001;

type

  IDTExtensibility2 = interface(IDispatch)
    ['{B65AD801-ABAF-11D0-BB8B-00A0C90F2744}']
    procedure OnConnection(const Application: IDispatch; ConnectMode: ext_ConnectMode;
                           const AddInInst: IDispatch; var custom: PSafeArray); safecall;
    procedure OnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); safecall;
    procedure OnAddInsUpdate(var custom: PSafeArray); safecall;
    procedure OnStartupComplete(var custom: PSafeArray); safecall;
    procedure OnBeginShutdown(var custom: PSafeArray); safecall;
  end;


  TMSOfficeAddIn = class(TAutoObject, IDTExtensibility2)
  private
  	function GetHostAppProperty(const PropName: PWideChar): OleVariant;
    function GetHostAppName(): String;
    function GetHostAppCommandBars(): Boolean;

		// IDTExtensibility2 methods:
    procedure OnConnection(const Application: IDispatch; ConnectMode: ext_ConnectMode;
                           const AddInInst: IDispatch; var custom: PSafeArray); safecall;
    procedure OnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); safecall;
    procedure OnAddInsUpdate(var custom: PSafeArray); safecall;
    procedure OnStartupComplete(var custom: PSafeArray); safecall;
    procedure OnBeginShutdown(var custom: PSafeArray); safecall;

  protected
  	FHostApp: IDispatch;
    FCommandBars: CommandBars;
    FHostAppName: String;

    // overridables:
    procedure DoOnConnection(ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
    	var custom: PSafeArray); virtual; safecall;
    procedure DoOnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); virtual; safecall;
    procedure DoOnAddInsUpdate(var custom: PSafeArray); virtual; safecall;
    procedure DoOnStartupComplete(var custom: PSafeArray); virtual; safecall;
    procedure DoOnBeginShutdown(var custom: PSafeArray); virtual; safecall;

  public

    // overridables:
    class procedure DoOnRegister(); virtual;
    class procedure DoOnUnregister(); virtual;

  end;

  TMSOfficeAddInClass = class of TMSOfficeAddIn;

  TMSOAddInFactory = class(TAutoObjectFactory)
  private
  	FMSOApp: String;
    FFriendlyName: String;
    FDescription: String;

    FLoadBehaviour: Integer;
    procedure AddKeys;
    procedure RemoveKeys;

  public

  	// Hide parent's one:
    constructor Create(const MSOApp, FriendlyName, Description: String;
    	const LoadBehaviour: Integer;	ComServer: TComServerObject;
      AutoClass: TAutoClass; const ClassID: TGUID; Instancing: TClassInstancing;
      ThreadingModel: TThreadingModel);

    procedure UpdateRegistry(Register: Boolean); override;

  end;

//******************************************************************************
//******************************************************************************

implementation

uses Windows, Registry, SysUtils, MSOAddIn_Defs;

//******************************************************************************
//******************************************************************************

{{*** VK ***********************************************************************
												TMSOAddInFactory
*******************************************************************************}


{{*** VK ***********************************************************************
  Procedure  :DoOnAddInsUpdate
  Purpose    :called from OnAddInsUpdate event handler
  Parameters :custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.DoOnAddInsUpdate(var custom: PSafeArray);
begin
	MessageBox( 0, 'TMSOfficeAddIn.DoOnAddInsUpdate', 'Message', 0 );
end;

{{*** VK ***********************************************************************
  Procedure  :DoOnBeginShutdown
  Purpose    :called from OnBeginShutdown event handler
  Parameters :custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.DoOnBeginShutdown(var custom: PSafeArray);
begin
	MessageBox( 0, 'TMSOfficeAddIn.DoOnBeginShutdown', 'Message', 0 );
end;

{{*** VK ***********************************************************************
  Procedure  :DoOnConnection
  Purpose    :called from OnConnection event handler
  Parameters :ConnectMode - connection mode; AddInInst - addin instance
*******************************************************************************}
procedure TMSOfficeAddIn.DoOnConnection(ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
  var custom: PSafeArray);
begin
	MessageBox( 0, 'TMSOfficeAddIn.DoOnConnection', 'Message', 0 );
end;

{{*** VK ***********************************************************************
  Procedure  :DoOnDisconnection
  Purpose    :called from OnDisconnection event handler
  Parameters :RemoveMode - removal mode; custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.DoOnDisconnection(RemoveMode: ext_DisconnectMode;
  var custom: PSafeArray);
begin
	MessageBox( 0, 'TMSOfficeAddIn.DoOnDisconnection', 'Message', 0 );
end;

{{*** VK ***********************************************************************
  Procedure  :DoOnStartupComplete
  Purpose    :called from OnStartupComplete event handler
  Parameters :custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.DoOnStartupComplete(var custom: PSafeArray);
begin
	MessageBox( 0, 'TMSOfficeAddIn.DoOnStartupComplete', 'Message', 0 );
end;

{{*** VK ***********************************************************************
  Function   :GetHostAppProperty
  Purpose    :retrieves host application property
  Parameters :PropName - property name
  Returns    :property value
*******************************************************************************}
function TMSOfficeAddIn.GetHostAppProperty(const PropName: PWideChar): OleVariant;
const
  IID_NULL: TGUID = '{00000000-0000-0000-0000-000000000000}';
var
	DispID: Integer;
  HRet: HResult;
	dispparamsNoArgs: DISPPARAMS;
begin

	try

  	HRet := Self.FHostApp.GetIDsOfNames( IID_NULL, @PropName, 1, LOCALE_USER_DEFAULT, @DispID );
		OleCheck( HRet );

    FillChar( dispparamsNoArgs, sizeof(DISPPARAMS), 0 );

  	HRet := Self.FHostApp.Invoke( DispID, IID_NULL, LOCALE_USER_DEFAULT,
    	DISPATCH_PROPERTYGET, dispparamsNoArgs, @Result, nil, nil );
		OleCheck( HRet );

  except

    on E: Exception do
    begin
      {$IFDEF DEBUG}
      OutputDebugString(PChar(E.Message));
      {$ENDIF}
      VariantInit( Result );
    end;
  end;

end;

{{*** VK ***********************************************************************
  Function   :GetHostAppName
  Purpose    :gets host application name
  Parameters :
  Returns    :host application name
*******************************************************************************}
function TMSOfficeAddIn.GetHostAppName(): String;
begin

	Result := Self.GetHostAppProperty('Name');
  if Result = '' then
  	Result := 'Unknown MS Office host application';

end;

{{*** VK ***********************************************************************
  Function   :GetHostAppCommandBars
  Purpose    :gets interface to host application command bars
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TMSOfficeAddIn.GetHostAppCommandBars: Boolean;
var
  TmpDisp: IDispatch;
begin

  try
    TmpDisp := Self.GetHostAppProperty('CommandBars');
    Self.FCommandBars := TmpDisp as CommandBars;
  except
    {$IFDEF DEBUG}
    OutputDebugString('TMSOfficeAddIn.GetHostAppCommandBars - failed to get command bars'#10);
    {$ENDIF}
  	Self.FCommandBars := nil;
  end;

  Result := Self.FCommandBars <> nil;

end;

{{*** VK ***********************************************************************
  Procedure  :OnAddInsUpdate
  Purpose    :OnAddInsUpdate event handler
  Parameters :custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.OnAddInsUpdate(var custom: PSafeArray);
begin
	Self.DoOnAddInsUpdate( custom );
end;

{{*** VK ***********************************************************************
  Procedure  :OnBeginShutdown
  Purpose    :Office addin OnBeginShutdown event handler
  Parameters :custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.OnBeginShutdown(var custom: PSafeArray);
begin
	Self.DoOnBeginShutdown( custom );
end;

{{*** VK ***********************************************************************
  Procedure  :OnConnection
  Purpose    :Office addin OnConnection event handler
  Parameters :
*******************************************************************************}
procedure TMSOfficeAddIn.OnConnection(const Application: IDispatch;
  ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
  var custom: PSafeArray);
begin

	Self.FHostApp := Application;
  Self.FHostAppName := Self.GetHostAppName();

  {{*** VK ************************************
    Outlook has different object model for command
    bars.
  ********************************************}
  if Self.FHostAppName <> 'Outlook' then
  begin
  	if not Self.GetHostAppCommandBars() then
    begin
			MessageBox( 0, 'Failed to bind to MS Office application command bars!', nil, 0 );
    end;
  end;

  Self.DoOnConnection( ConnectMode, AddInInst, custom );

end;

{{*** VK ***********************************************************************
  Procedure  :OnDisconnection
  Purpose    :Office addin OnDisconnection event handler
  Parameters :RemoveMode - remove mode; custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.OnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray);
begin
	Self.DoOnDisconnection( RemoveMode, custom );
end;

{{*** VK ***********************************************************************
  Procedure  :OnStartupComplete
  Purpose    :Office addin OnStartupComplete event handler
  Parameters :custom - custom data
*******************************************************************************}
procedure TMSOfficeAddIn.OnStartupComplete(var custom: PSafeArray);
begin
	Self.DoOnStartupComplete( custom );
end;

{{*** VK ***********************************************************************
  Procedure  :DoOnRegister
  Purpose    :Routine to perform post-registration tasks. Will be called after
              CoClass registered by class factory.
  Parameters :
*******************************************************************************}
class procedure TMSOfficeAddIn.DoOnRegister;
begin
  ;
end;

{{*** VK ***********************************************************************
  Procedure  :DoOnUnregister
  Purpose    :Routine to perform post-unregistration tasks. Will be called after
              CoClass unregistered by class factory.
  Parameters :
*******************************************************************************}
class procedure TMSOfficeAddIn.DoOnUnregister;
begin
  ;
end;

{{*** VK ***********************************************************************
												TMSOAddInFactory
*******************************************************************************}


{{*** VK ***********************************************************************
  Procedure  :TMSOAddInFactory.Create
  Purpose    :MSOAddIn Factory constructor.
  Parameters :MSOApp - name of MS Office application ('Word', 'Excel', 'Outlook');
    FriendlyName - add-in friendly name; Description - add-in description;
    LoadBehaviour - determines how add-in will be loaded by Office application:
    Disconnected - 0, Connected - 1, BootLoad - 2, DemandLoad - 8,
    ConnectFirstTime - 16
*******************************************************************************}
constructor TMSOAddInFactory.Create(const MSOApp, FriendlyName, Description: String;
  const LoadBehaviour: Integer; ComServer: TComServerObject;
  AutoClass: TAutoClass; const ClassID: TGUID;
  Instancing: TClassInstancing; ThreadingModel: TThreadingModel);
begin

  FMSOApp := MSOApp;
  FFriendlyName := FriendlyName;
  FDescription := Description;
  FLoadBehaviour := LoadBehaviour;

	inherited Create(ComServer, AutoClass, ClassID,
    Instancing, ThreadingModel);

end;

{{*** VK ***********************************************************************
  Procedure  :AddKeys
  Purpose    :adds add-in related keys to registry
  Parameters :
*******************************************************************************}
procedure TMSOAddInFactory.AddKeys;
begin

  with TRegistry.Create() do
  try

    RootKey := HKEY_CURRENT_USER;
    if OpenKey( Format( ADDIN_KEY, [FMSOApp, Self.ProgID] ), True) then
    begin

			WriteString( 'FriendlyName', FFriendlyName );
			WriteString( 'Description', FDescription );
      WriteInteger( 'LoadBehavior', FLoadBehaviour );

    	CloseKey;
    end;

  finally
    Free();
  end;// try-finally

  // Perform post-registration tasks:
  TMSOfficeAddInClass(Self.ComClass).DoOnRegister();

end;

{{*** VK ***********************************************************************
  Procedure  :RemoveKeys
  Purpose    :removes add-in related keys from registry
  Parameters :
*******************************************************************************}
procedure TMSOAddInFactory.RemoveKeys;
begin

  with TRegistry.Create() do
  try
    RootKey := HKEY_CURRENT_USER;
    DeleteKey( Format( ADDIN_KEY, [FMSOApp, Self.ProgID] ) );
  finally
    Free();
  end;// try-finally

  // Perform post-unregistration tasks:
  TMSOfficeAddInClass(Self.ComClass).DoOnUnregister();

end;

{{*** VK ***********************************************************************
  Procedure  :UpdateRegistry
  Purpose    :TAutoObjectFactory UpdateRegistry method override
  Parameters :
*******************************************************************************}
procedure TMSOAddInFactory.UpdateRegistry(Register: Boolean);
begin

  inherited;

  if Register then
  	Self.AddKeys()
  else
  	Self.RemoveKeys();

end;

end.
