unit MainSkin;

// Version 1.1
// Date: xxxx of January 2003
//
// Michel Verbraak (miv@xs4all.nl)
// http://miv.xs4all.nl/delphi

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Graphics,
  CommCtrl, Types;

type
  TMainSkin = class(TCustomControl)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FRegion : hrgn;
    Function GetRegion: hrgn; Virtual;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; Override;
    Procedure DeleteRegion;
    Property Region: hrgn Read GetRegion;
  published
    { Published declarations }
  end;

  TParentSkin = class( TMainSkin )
  private
    { Private declarations }
  protected
    { Protected declarations }
    Procedure SetParent(Value:TWinControl); override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Procedure Paint; Override;
  published
    Property OnClick;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
  End;

  TObjectSkin = class(TMainSkin)
  private
    { Private declarations }
  protected
    { Protected declarations }
    Procedure SetParent(Value:TWinControl); override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
  end;

  TRoundRectSkinPart = class(TObjectSkin)
  private
    { Private declarations }
    FCircleWidth,
    FCircleHeight : Integer;
  protected
    { Protected declarations }
    Procedure SetCircleWidth( Value: Integer);
    Procedure SetCircleHeight( Value: Integer);
    Function GetRegion: hrgn; Override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Procedure Paint; Override;
  published
    { Published declarations }
    Property CircleWidth: Integer Read FCircleWidth Write SetCircleWidth;
    Property CircleHeight : Integer Read FCircleHeight Write SetCircleHeight;
  end;

  TPolygonSkinPart = class(TObjectSkin)
  private
    { Private declarations }
  protected
    { Protected declarations }
    Function GetRegion: hrgn; Override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Procedure Paint; Override;
  published
    { Published declarations }
  end;

  TPolygonPoint = Class(TCustomControl)
  Protected
    Procedure SetParent(Value:TWinControl); override;
    Procedure WMMove(var Message: TMessage); message WM_MOVE;
  Public
    Constructor Create( AOwner: TComponent); Override;
    Destructor Destroy; Override;
    Procedure Paint; Override;
  End;

  TBitmapSkinPart = class(TObjectSkin)
  private
    { Private declarations }
    FGlyph : TBitmap;
    FTransparentColor : TColor;
    FShowImage : Boolean;
  protected
    { Protected declarations }
    Function GetRegion: hrgn; Override;
    Procedure SetGlyph(Value: TBitmap);
    Procedure SetTransparentColor(Value: TColor);
    Procedure SetShowImage(Value: Boolean);
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; Override;
    Procedure Paint; Override;
  published
    { Published declarations }
    Property Glyph: TBitmap Read FGlyph Write SetGlyph;
    Property TransparentColor: TColor Read FTransparentColor Write SetTransparentColor;
    Property ShowImage: Boolean Read FShowImage Write SetShowImage;

    Property OnClick;
    Property OnDblClick;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
  end;

procedure Register;

implementation

{$R 'MainSkin.dcr'}

// Parent class of Skin classes.
//

Function TMainSkin.GetRegion: hrgn;

Begin
  Result := FRegion;
End;

Constructor TMainSkin.Create( AOwner: TComponent);

Begin
  Inherited Create( AOwner );
  ControlStyle := ControlStyle + [csAcceptsControls];
  FRegion := 0;
End;

Destructor TMainSkin.Destroy;

Begin
  If FRegion <> 0 Then DeleteObject( FRegion );
  Inherited Destroy;
End;

Procedure TMainSkin.DeleteRegion;

Begin
  If FRegion = 0 Then Exit;
  DeleteObject( FRegion );
  FRegion := 0;
End;

// Class where Skin parts are dropped on.
// This component merges all children skin parts into one region.
//
Procedure TParentSkin.SetParent(Value:TWinControl);

Begin
  Inherited;
  If Value <> nil Then
    If not (Value Is TForm) Then
      Begin
        Raise Exception.Create ('Drop the ParentSkin on a FORM!');
      End
    Else
      Begin
        TForm(Value).borderstyle := bsNone;
        Top := 0;
        Left := 0;
        Width := TForm(Value).ClientWidth;
        Height := TForm(Value).ClientHeight;
      End;
End;

Constructor TParentSkin.Create(AOwner: TComponent);

Begin
  Inherited Create(AOwner);
End;

Function MoveControls( ToParent: TWinControl; SkinPart: TObjectSkin): Boolean;

Var
  Counter : Integer;

Begin
  Counter := 0;
  Result := False;
  While Counter < SkinPart.ControlCount Do
    If (Not( SkinPart.Controls[Counter] is TObjectSkin ) And
        Not( SkinPart.Controls[Counter] is TPolygonPoint )) Then
      Begin
        SkinPart.Controls[Counter].Left := SkinPart.Controls[Counter].Left + SkinPart.Left;
        SkinPart.Controls[Counter].Top := SkinPart.Controls[Counter].Top + SkinPart.Top;
        SkinPart.Controls[Counter].Parent := ToParent;
        Result := True;
      End
    Else
      Counter := Counter + 1;
End;

procedure TParentSkin.Paint;

Var
  Counter : Integer;
  Result : Integer;

begin
  If Region <> 0 Then Exit;

  if not (csDesigning in ComponentState) then
    Begin
      If ControlCount > 0 Then
        Begin
          Counter := 0;
          // Move controls on skinpart to parentskin.
          While Counter < ControlCount Do
            Begin
              If (Controls[Counter] Is TObjectSkin) Then
                Begin
                  If MoveControls( Self, TObjectSkin(Controls[Counter]) ) Then
                    Counter := 0
                  Else
                    Counter := Counter + 1;
                End
              Else
                Counter := Counter + 1;
            End;
          // Create Region.
          FRegion := CreateRectRgn(0,0, 0,0);
          Counter := 0;
          While Counter < ControlCount Do
            Begin
              If (Controls[Counter] Is TObjectSkin) And
                 (TMainSkin( Controls[Counter] ).Region <> 0) Then
                Begin
                  Result := CombineRgn(FRegion, Region, TMainSkin( Controls[Counter] ).Region,RGN_OR);
                  If Result = Error Then
                    Exit;
                End;
              Counter := Counter + 1;
            End;
        End;
      SetWindowRgn( Parent.handle, FRegion, true );
    End;
end;

// Parent class of Skin (region) parts.
// These must be dropped on a ParentSkin

Procedure TObjectSkin.SetParent(Value:TWinControl);

Begin
  Inherited;
  If Value <> nil Then
    If not (Value Is TParentSkin) Then
      Begin
        Raise Exception.Create ('Drop the Skin Part on a ParentSkin!');
      End;
End;

Constructor TObjectSkin.Create(AOwner: TComponent);

Begin
  Inherited Create(AOwner);
  Visible := False;
End;

// With this component a Round Rectengular regionpart can
// be created.
// Examples : Square = CircleHeight and CircleWidth equals 0
//            Circle = CircleHeight and CircleWidth equals height div 2
//            Ellipses = figure out ourself.

Procedure TRoundRectSkinPart.SetCircleWidth( Value: Integer);

Begin
  If Value = FCircleWidth Then Exit;

  FCircleWidth := Value;
  DeleteRegion;
  FRegion := CreateRoundRectRgn( Left ,Top ,Left + ClientWidth, Top + ClientHeight, CircleWidth, CircleHeight);
  RePaint;
End;

Procedure TRoundRectSkinPart.SetCircleHeight( Value: Integer);

Begin
  If Value = FCircleHeight Then Exit;

  FCircleHeight := Value;
  DeleteRegion;
  FRegion := CreateRoundRectRgn( Left ,Top ,Left + ClientWidth, Top + ClientHeight, CircleWidth, CircleHeight);
  RePaint;
End;

Function TRoundRectSkinPart.GetRegion: hrgn;

Begin
  If FRegion = 0 Then
    FRegion := CreateRoundRectRgn( Left ,Top ,Left + ClientWidth, Top + ClientHeight, CircleWidth, CircleHeight);

  Result := FRegion;
End;

Constructor TRoundRectSkinPart.Create( AOwner: TComponent);

Begin
  Inherited Create( AOwner );
  FCircleWidth := 50;
  FCircleHeight := 50;
End;

procedure TRoundRectSkinPart.Paint;

begin
  if (csDesigning in ComponentState) then
    Begin
      With Canvas Do
        Begin
          RoundRect( 0, 0, ClientWidth, ClientHeight, FCircleWidth, FCircleHeight);
        End;
    End;
end;

// With this component polygon region part can
// be created.
// Examples: Stars, etc....
// The polygon is made up of PolygonPoints which have to be
// dropped onto this object. A minimum of three points is
// required.
//
// Problems to be fixed:
// The array that is build with all the points can not be dynamic.
// Therefore the amount of points is fixed. Currently 100.
// If someone knows how to solve this please let me know.

Function TPolygonSkinPart.GetRegion: hrgn;

Var
  TmpArray : Array[0..100] of Types.TPoint;
  ArrayCounter : Integer;
  Counter : Integer;
  First : Integer;

Begin
  If FRegion <> 0 Then
    Begin
      Result := FRegion;
      Exit;
    End;

  If ControlCount <= 2 Then
    Begin
      FRegion := 0;
      Result := 0;
      Exit;
    End;

  First := -1;
  ArrayCounter := 0;
  For Counter := 0 To ControlCount -1 Do
    If Controls[Counter] is TPolygonPoint then
      Begin
        If First = -1 Then First := Counter;
        TmpArray[ Counter ].X := Controls[Counter].Left + Left;
        TmpArray[ Counter ].Y := Controls[Counter].Top + Top;
        ArrayCounter := ArrayCounter + 1;
      End;
  TmpArray[ ArrayCounter ].X := Controls[First].Left + Left;
  TmpArray[ ArrayCounter ].Y := Controls[First].Top + Top;

  FRegion := CreatePolygonRgn( TmpArray, ControlCount + 1, WINDING);
  Result := FREgion;
End;

Constructor TPolygonSkinPart.Create( AOwner: TComponent);

Begin
  Inherited Create( AOwner );
End;

procedure TPolygonSkinPart.Paint;

Var
  Counter : Integer;
  StartCounter : Integer;
  Rect : TRect;
  First : Integer;

begin
  if (csDesigning in ComponentState) And
     (ControlCount > 2) then
    Begin
      With Canvas Do
        Begin
          First := -1;
          StartCounter := 0;
          While (First = -1) And (StartCounter < ControlCount) Do
            Begin
              If Controls[StartCounter] is TPolygonPoint Then
                Begin
                  MoveTo( Controls[StartCounter].Left, Controls[StartCounter].Top);
                  First := StartCounter;
                  StartCounter := StartCounter + 1;
                End
              Else
                StartCounter := StartCounter + 1;
            End;
          For Counter := StartCounter To ControlCount - 1 Do
            If Controls[Counter] is TPolygonPoint Then
              LineTo(  Controls[Counter].Left, Controls[Counter].Top);
          If First <> -1 Then
            LineTo( Controls[First].Left, Controls[First].Top);
        End;
    End;
  If (csDesigning in ComponentState) Then
    Begin
      With Canvas Do
        Begin
          Pen.Style := psDashDot;
          Brush.Style := bsClear;
          Rect.Left := 0;
          Rect.Top := 0;
          Rect.Right := Width;
          Rect.Bottom := Height;
          Rectangle( Rect );
          Pen.Style := psSolid;
          Brush.Style := bsSolid;
        End;
    End;
end;

// This component is used to define a point for the
// polygoneskinobject region part.
//

Procedure TPolygonPoint.WMMove(var Message: TMessage);

Begin
  TPolygonSkinPart(Parent).RePaint;
End;

Procedure TPolygonPoint.SetParent(Value:TWinControl);

Var
  OldParent : TWinControl;

Begin
  OldParent := Nil;
  If Parent <> Nil Then OldParent := Parent;

  Inherited SetParent(Value);

  If Parent <> Nil Then OldParent := Parent;

  If Value <> nil Then
    If not (Value Is TPolygonSkinPart) Then
      Begin
        Raise Exception.Create ('Drop the Polygon Point on a PolygonSkinPart!');
      End;

  If OldParent <> Nil Then
    OldParent.Repaint;
End;

Constructor TPolygonPoint.Create( AOwner: TComponent);

Begin
  Inherited Create( AOwner );
  ControlStyle := ControlStyle - [csFramed];
  Enabled := False;
  Visible := False;
  Width := 10;
  Height := 10;
End;

Destructor TPolygonPoint.Destroy;

Begin
  Inherited Destroy;
End;

Procedure TPolygonPoint.Paint;

Var
  Rect : TRect;

Begin
  With Canvas Do
    Begin
      Brush.Color := clRed;
      Rect.Left := 0;
      Rect.Top := 0;
      Rect.Bottom := Height;
      Rect.Right := Width;
      FillRect(Rect);
    End;
End;

Function TBitmapSkinPart.GetRegion: hrgn;

Var
  X, Y: Integer;
  TmpRgn : hrgn;

Begin
  If FRegion <> 0 Then
    Begin
      Result := FRegion;
      Exit;
    End;
    
  FRegion := CreateRectRgn(0,0, 0,0);
  For Y:= 0 To Glyph.Height - 1 Do
    For X:= 0 To Glyph.Width - 1 Do
      Begin
        If Glyph.Canvas.Pixels[X,Y] <> TransparentColor Then
          Begin
            TmpRgn := CreateRectRgn(Left + X, Top + Y, Left + X+1,Top + Y+1);
            If CombineRgn(FRegion, Region, TmpRgn,RGN_OR) = ERROR Then
              Begin
                DeleteRegion;
                Result := 0;
                Exit;
              End;
            DeleteObject(TmpRgn);
          End;
      End;
  Result := FRegion;
End;

Procedure TBitmapSkinPart.SetGlyph(Value: TBitmap);

Begin
  FGlyph.Assign(Value);
End;

Procedure TBitmapSkinPart.SetTransparentColor(Value: TColor);

Begin
  If Value = FTransparentColor Then Exit;
  FTransparentColor := Value;
  DeleteRegion;
  RePaint;
End;

Procedure TBitmapSkinPart.SetShowImage(Value: Boolean);

Begin
  If Value = FShowImage Then Exit;
  FShowImage := Value;
  Visible := Value;
  Paint;
End;

Constructor TBitmapSkinPart.Create(AOwner: TComponent);

Begin
  Inherited Create(AOwner);
  FGlyph := Tbitmap.Create;
  FShowImage := true;
  FTransparentColor := clPurple;
  Visible := True;
End;

Destructor TBitmapSkinPart.Destroy;

Begin
  FGlyph.Free;
  Inherited Destroy;
End;

Procedure TBitmapSkinPart.Paint;

Var
  X, Y: Integer;

Begin
  With Canvas Do
    Begin
      Try
        Draw(0, 0, Glyph );
      Except
      End;

      If (csDesigning in ComponentState) Then
        Begin
          For Y:= 0 To Glyph.Height - 1 Do
            For X:= 0 To Glyph.Width - 1 Do
              If Pixels[X,Y] = TransparentColor Then
                Pixels[X,Y] := clBtnFace;
        End;
    End;
End;

procedure Register;
begin
  RegisterComponents('MIV Skins', [TParentSkin, TRoundRectSkinPart, TPolygonSkinPart, TPolygonPoint, TBitmapSkinPart]);
end;

end.
