{ TNormalBand, TNormalLabel -      
                                .
  (C) by Vladimir Kladov, 2002

  ,    ()     Delphi,
    TWinControl.        .
  (   rich-edit-,    
      ).
  ,      .     
       .

}
unit NormalBands;

interface

{$I DelphiVersions.inc}

uses
  Windows, Messages, SysUtils, Classes, Forms, Dialogs, Graphics, ExtCtrls, Controls,
  StdCtrls;

type
  TFrame = ( frLeft, frTop, frRigth, frBottom );
  TFrames = set of TFrame;

  TNormalBand = class( TPanel )
  private
    FFrames: TFrames;
    FFrameWidth: Integer;
    procedure SetFrames(const Value: TFrames);
    procedure SetFrameWidth(const Value: Integer);
  protected
    procedure Paint; override;
  public
    constructor Create( AOwner: TComponent ); override;
  published
    property Frames: TFrames read FFrames write SetFrames;
    property FrameWidth: Integer read FFrameWidth write SetFrameWidth;
  end;

  TNormalLabel = class( TNormalBand )
  private
    FFrames: TFrames;
    FFrameWidth: Integer;
    FLayout: TTextLayout;
    FWordWrap: Boolean;
    FAutosize: Boolean;
    procedure SetFrames(const Value: TFrames);
    procedure SetFrameWidth(const Value: Integer);
    procedure SetLayout(const Value: TTextLayout);
    procedure SetWordWrap(const Value: Boolean);
    procedure Set_Autosize(const Value: Boolean);
    procedure SetCaption(const Value: String);
    function GetCaption: String;
    function GetParentFont: Boolean;
    procedure SetParentFont(const Value: Boolean);
  protected
    procedure Paint; override;
    function CanAutoSize( var Width, Height: Integer ): Boolean; override;
    procedure Resize; override;
    procedure DoAutosize;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  public
    constructor Create( AOwner: TComponent ); override;
    property Canvas;
  published
    property Frames: TFrames read FFrames write SetFrames;
    property FrameWidth: Integer read FFrameWidth write SetFrameWidth;
    property Alignment;
    property Layout: TTextLayout read FLayout write SetLayout;
    property WordWrap: Boolean read FWordWrap write SetWordWrap;
    property Autosize: Boolean read FAutosize write Set_Autosize;
    property Caption: String read GetCaption write SetCaption;
    property ParentFont: Boolean read GetParentFont write SetParentFont;
  end;

implementation

{ TNormalBand }

constructor TNormalBand.Create(AOwner: TComponent);
begin
  inherited;
  Color := clWhite;
  Font.Color := clBlack;
  Font.Name := 'Arial';
  Font.Size := 10;
  BorderStyle := bsNone;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Caption := ' ';
  FFrameWidth := 1;
end;

procedure TNormalBand.Paint;
begin
  inherited;
  Canvas.Brush.Color := Font.Color;
  if frLeft in Frames then
    Canvas.FillRect( Rect( 0, 0, FrameWidth, Height ) );
  if frTop in Frames then
    Canvas.FillRect( Rect( 0, 0, Width, FrameWidth ) );
  if frRigth in Frames then
    Canvas.FillRect( Rect( Width - FrameWidth, 0, Width, Height ) );
  if frBottom in Frames then
    Canvas.FillRect( Rect( 0, Height - FrameWidth, Width, Height ) );
end;

procedure TNormalBand.SetFrames(const Value: TFrames);
begin
  FFrames := Value;
  Invalidate;
end;

procedure TNormalBand.SetFrameWidth(const Value: Integer);
begin
  FFrameWidth := Value;
  Invalidate;
end;

{ TNormalLabel }

function TNormalLabel.CanAutoSize(var Width, Height: Integer): Boolean;
var R, R1: TRect;
    F: DWORD;
begin
  Result := FALSE;
  if not FAutosize then Exit;
  if Trim( Caption ) <> '' then
  begin
    Canvas.Font.Assign( Font );
    F := 0;
    case Layout of
    tlTop:    F := DT_TOP;
    tlCenter: F := DT_VCENTER;
    tlBottom: F := DT_BOTTOM;
    end;
    case Alignment of
    taLeftJustify:   F := F or DT_LEFT;
    taCenter:        F := F or DT_CENTER;
    taRightJustify:  F := F or DT_RIGHT;
    end;
    case WordWrap of
    FALSE:    F := F or DT_SINGLELINE;
    TRUE:     F := F or DT_WORDBREAK;
    end;
    R := ClientRect;
    if Frames <> [] then
      InflateRect( R, -1, -1 );
    R1 := R;
    Windows.DrawText( Canvas.Handle, PChar( Caption ), Length( Caption ),
                      R1, F or DT_CALCRECT );
    if not (Align in [ alTop, alBottom, alClient ]) then
      Width := R1.Right - R1.Left;
    if not (Align in [ alLeft, alRight, alClient ]) then
      Height := R1.Bottom - R1.Top;
  end;
  Result := (Width <> Self.Width) or (Height <> Self.Height);
end;

procedure TNormalLabel.CMFontChanged(var Message: TMessage);
begin
  inherited;
  DoAutosize;
end;

constructor TNormalLabel.Create(AOwner: TComponent);
begin
  inherited;
  if (AOwner <> nil) and (AOwner is TForm) and
     ((AOwner as TForm).Color <> clWhite) then
  begin
    Color := clWhite;
    Font.Color := clBlack;
    Font.Name := 'Arial';
    Font.Size := 10;
  end;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  FFrameWidth := 1;
  inherited ParentFont := TRUE;
  Autosize := TRUE;
end;

procedure TNormalLabel.DoAutosize;
var W, H: Integer;
begin
  if csLoading in ComponentState then Exit;
  if Align = alClient then Exit;
  W := Width;
  H := Height;
  if CanAutoSize( W, H ) then
  begin
    Width := W;
    Height := H;
  end;
end;

function TNormalLabel.GetCaption: String;
begin
  Result := inherited Caption;
end;

function TNormalLabel.GetParentFont: Boolean;
begin
  Result := inherited ParentFont;
end;

procedure TNormalLabel.Paint;
var F: DWORD;
    R, R1: TRect;
begin
  Canvas.Brush.Color := Color;
  Canvas.FillRect( ClientRect );
  Canvas.Font.Assign( Font );
  if Trim( Caption ) <> '' then
  begin
    F := 0;
    R := ClientRect;
    if Frames <> [] then
      InflateRect( R, -1, -1 );
    case Layout of
    tlTop:    F := DT_TOP;
    tlCenter: F := DT_VCENTER;
    tlBottom: F := DT_BOTTOM;
    end;
    case Alignment of
    taLeftJustify:   F := F or DT_LEFT;
    taCenter:        F := F or DT_CENTER;
    taRightJustify:  F := F or DT_RIGHT;
    end;
    case WordWrap of
    FALSE: begin
             F := F or DT_SINGLELINE;
             if (Alignment <> taLeftJustify) and
                (R.Right - R.Left <= Canvas.TextWidth( Caption )) then
               F := F and not (DT_CENTER or DT_RIGHT);
           end;
    TRUE:  F := F or DT_WORDBREAK;
    end;
    if Layout <> tlTop then
    begin
      R1 := R;
      Windows.DrawText( Canvas.Handle, PChar( Caption ), Length( Caption ),
                        R1, F or DT_CALCRECT );
      if R1.Bottom - R1.Top < R.Bottom - R.Top then
      begin
        if Layout = tlCenter then
          OffsetRect( R, 0, ((R.Bottom - R.Top) - (R1.Bottom - R1.Top)) div 2 )
        else
          OffsetRect( R, 0, (R.Bottom - R.Top) - (R1.Bottom - R1.Top) );
      end;
      Windows.DrawText( Canvas.Handle, PChar( Caption ), Length( Caption ),
                        R, F );
    end
      else
    Windows.DrawText( Canvas.Handle, PChar( Caption ), Length( Caption ),
                      R, F );
  end;
  Canvas.Brush.Color := Font.Color;
  if frLeft in Frames then
    Canvas.FillRect( Rect( 0, 0, FrameWidth, Height ) );
  if frTop in Frames then
    Canvas.FillRect( Rect( 0, 0, Width, FrameWidth ) );
  if frRigth in Frames then
    Canvas.FillRect( Rect( Width - FrameWidth, 0, Width, Height ) );
  if frBottom in Frames then
    Canvas.FillRect( Rect( 0, Height - FrameWidth, Width, Height ) );
end;

procedure TNormalLabel.Resize;
var W, H: Integer;
begin
  W := Width;
  H := Height;
  if CanAutoSize( W, H ) then
  begin
    Self.Width := W;
    Self.Height := H;
  end
    else
    inherited;
end;

procedure TNormalLabel.Set_Autosize(const Value: Boolean);
begin
  FAutosize := Value;
  DoAutosize;
end;

procedure TNormalLabel.SetCaption(const Value: String);
begin
  inherited Caption := Value;
  DoAutoSize;
end;

procedure TNormalLabel.SetFrames(const Value: TFrames);
begin
  FFrames := Value;
  Invalidate;
end;

procedure TNormalLabel.SetFrameWidth(const Value: Integer);
begin
  FFrameWidth := Value;
  Invalidate;
end;

procedure TNormalLabel.SetLayout(const Value: TTextLayout);
begin
  if FLayout = Value then Exit;
  FLayout := Value;
  Invalidate;
end;

procedure TNormalLabel.SetParentFont(const Value: Boolean);
begin
  inherited ParentFont := Value;
  DoAutosize;
end;

procedure TNormalLabel.SetWordWrap(const Value: Boolean);
begin
  if FWordWrap = Value then Exit;
  FWordWrap := Value;
  Invalidate;
end;

end.
