{-----------------------------------------------------------------------------
 Unit Name: OutlookAddIn_IMPL

 Author:    Vadim Krochak
 Copyright:  2003 All rights reserved.
 E.Mail:    vkrochak@bezeqnt.net
 License:   Don't redistribute for cash.  Don't change and
  redistribute under your name.  Let me know if you fix anything or improve
  anything.

 Purpose: TOutlookAddIn implementation
 History:
-----------------------------------------------------------------------------}

unit OutlookAddIn_IMPL;

interface

uses ComObj, ActiveX, MSOfficeAddIn, MSOAddIn_TLB, Outlook_TLB, Office_TLB, MSOEventSink, StdVcl;

type

  TOutlookAddIn = class(TMSOfficeAddIn, IOutlookAddIn)
  private

  	FOutlookApp: OutlookApplication;
    FCommandBar: CommandBar;

    FCommandBarBtn: CommandBarButton;
    FBtnEventSink: TCommandBarButtonEventsSink;

    function IsCmdBarExists(): Boolean;
    procedure OnBtnClick(const Ctrl: CommandBarButton; var CancelDefault: WordBool);

  protected

    procedure DoOnConnection(ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
    	var custom: PSafeArray); override;
    procedure DoOnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); override;
    procedure DoOnAddInsUpdate(var custom: PSafeArray); override;
    procedure DoOnStartupComplete(var custom: PSafeArray); override;
    procedure DoOnBeginShutdown(var custom: PSafeArray); override;

    function BindToHost(): Boolean;
    function UnbindFromHost(): Boolean;

  end;

//******************************************************************************
//******************************************************************************

implementation

uses Windows, ComServ, SysUtils, Controls, MSOAddIn_Defs, Form_AddrInfo_U, MSOAddIn_Utils;

//******************************************************************************
//******************************************************************************


procedure TOutlookAddIn.DoOnAddInsUpdate(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TOutlookAddIn.DoOnAddInsUpdate'#10);
  {$ENDIF}
end;

//******************************************************************************
procedure TOutlookAddIn.DoOnBeginShutdown(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TOutlookAddIn.DoOnBeginShutdown'#10);
  {$ENDIF}
end;

//******************************************************************************
procedure TOutlookAddIn.DoOnConnection(ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
  var custom: PSafeArray);
begin

	try
	  FOutlookApp := Self.FHostApp as OutlookApplication;
  except
    on E: Exception do
    begin
			{$IFDEF DEBUG}
      OutputDebugString(PChar(E.Message));
      {$ENDIF}
			Exit;
    end;
  end;// try-except

  Self.BindToHost();

end;

//******************************************************************************
procedure TOutlookAddIn.DoOnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray);
begin
 	Self.UnbindFromHost();

  // removed by user from COM add-in manager:
  if RemoveMode = ext_dm_UserClosed then
  begin
    Self.FCommandBar.Delete();        
  end; // if

end;

//******************************************************************************
procedure TOutlookAddIn.DoOnStartupComplete(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TOutlookAddIn.DoOnStartupComplete'#10);
  {$ENDIF}
end;

{{*** VK ***********************************************************************
  Function   :TOutlookAddIn.BindToHost
  Purpose    :binds addin to Outlook, creates command bar, etc.
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TOutlookAddIn.BindToHost: Boolean;
begin

	try

  	{***(V.K.)******************************
      in Outlook, CommandBars is 'hosted' by
			ActiveExplorer, not by Application as in
      rest of MS Office aplications.
    ***************************************}
    Self.FCommandBars := Self.FOutlookApp.ActiveExplorer.CommandBars;

    if Self.IsCmdBarExists() then
    begin
      //(VK): find existing button:
      FCommandBarBtn := Self.FCommandBar.FindControl( msoControlButton, 1, OUTLOOKADDIN_BTN_NAME + TAG_SUFFIX, 1, 1 )  as CommandBarButton;
    end
    else
    begin

    	//(VK): add command bar:
      Self.FCommandBar := Self.FCommandBars.Add( OUTLOOKADDIN_CMD_BAR_NAME, msoBarTop, 0, False );

			//(VK): add button:
      Self.FCommandBarBtn := Self.FCommandBar.Controls.Add( msoControlButton, 1, 0, 1, False ) as CommandBarButton;

      Self.FCommandBarBtn.Set_Tag( OUTLOOKADDIN_BTN_NAME + TAG_SUFFIX );
      Self.FCommandBarBtn.Set_TooltipText( OUTLOOKADDIN_BTN_NAME );
      Self.FCommandBarBtn.Set_Caption( OUTLOOKADDIN_BTN_NAME );

      // user should see newly created command bar:
      Self.FCommandBar.Set_Visible( True );

    end;// if-else

  except
    on E: Exception do
    begin
			{$IFDEF DEBUG}
      OutputDebugString(Pchar(E.Message));
      {$ENDIF}
			Result := False; Exit;
    end;
  end;// try-except


  //(VK): 'connect' button to event sink:
  if Self.FBtnEventSink = nil then
  	Self.FBtnEventSink := TCommandBarButtonEventsSink.Create(DIID__CommandBarButtonEvents);

	Self.FBtnEventSink.OnCommandBarButtonClick := Self.OnBtnClick;

	Result := Self.FBtnEventSink.Connect( Self.FCommandBarBtn );

end;

{{*** VK ***********************************************************************
  Function   :TOutlookAddIn.UnbindFromHost
  Purpose    :unbinds addin from outlook
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TOutlookAddIn.UnbindFromHost: Boolean;
begin
 	Result := Self.FBtnEventSink.Disconnect( Self.FCommandBarBtn );
  Self.FBtnEventSink := nil;
end;

{{*** VK ***********************************************************************
  Function   :TOutlookAddIn.IsCmdBarExists
  Purpose    :checks if named command bar exists
  Parameters :CmdBarName - name of command bar to check
  Returns    :true if named command bar exists
*******************************************************************************}
function TOutlookAddIn.IsCmdBarExists(): Boolean;
var
	BarIndex: Integer;
begin

  BarIndex := MSOAddIn_Utils.FindCommandBar( Self.FCommandBars, OUTLOOKADDIN_CMD_BAR_NAME );
  Result := BarIndex > 0;
  if Result then
  begin
    Self.FCommandBar := Self.FCommandBars.Item[BarIndex];
  end; // if

end;

{{*** VK ***********************************************************************
  Procedure  :TOutlookAddIn.OnBtnClick
  Purpose    :VCL event handler
  Parameters :
*******************************************************************************}
procedure TOutlookAddIn.OnBtnClick(const Ctrl: CommandBarButton; var CancelDefault: WordBool);
var
	Form_AddrInfo: TForm_AddrInfo;
  NewContact: ContactItem;
begin

	Form_AddrInfo := TForm_AddrInfo.Create(nil);
  if Form_AddrInfo.ShowModal = mrOK then
  begin

  	try

    	NewContact :=  Self.FOutlookApp.CreateItem( olContactItem ) as ContactItem;

      NewContact.Set_FirstName( Form_AddrInfo.Edit_Name.Text );
      NewContact.Set_HomeTelephoneNumber( Form_AddrInfo.Edit_Phone.Text );
      NewContact.Set_Email1Address( Form_AddrInfo.Edit_Email.Text );

      NewContact.Save;

			Self.FOutlookApp.ActiveExplorer.Set_CurrentFolder(
      	Self.FOutlookApp.GetNamespace( OUTLOOK_MAPI ).GetDefaultFolder(olFolderContacts)
      );

    except
      on E: Exception do
      begin
        {$IFDEF DEBUG}
        OutputDebugString(PChar(E.Message));
        {$ENDIF}
      end;
    end;// try-except

  end;// if

	Form_AddrInfo.Free;

end;

//******************************************************************************
//******************************************************************************

initialization

  TMSOAddInFactory.Create( OUTLOOK_APP, OUTLOOKADDIN_NAME,
    OUTLOOKADDIN_DESCRIPTION, $03 {'BootLoad' or 'Connected'},
    ComServer, TOutlookAddIn, Class_OutlookAddIn,
    ciMultiInstance, tmApartment
  );

end.
