{-------------------------------------------------------------------------------
 Unit Name: WordAddIn_IMPL

 Author:    Vadim Krochak
 Copyright:  2003 All rights reserved.
 E.Mail:    vkrochak@bezeqnt.net
 License:   Don't redistribute for cash.  Don't change and
  redistribute under your name.  Let me know if you fix anything or improve
  anything.

 Purpose: TWordAddIn (MS Word addin) implementation
 History: 
-------------------------------------------------------------------------------}

unit WordAddIn_IMPL;

interface

uses
  ComObj, ActiveX, MSOAddIn_TLB, MSOfficeAddIn, Word_TLB, Office_TLB, MSOEventSink, StdVcl;

type

  TWordAddIn = class(TMSOfficeAddIn, IWordAddIn)
  private
    FCommandBar: CommandBar;

    FCommandBarBtn: CommandBarButton;
    FBtnEventSink: TCommandBarButtonEventsSink;

    function IsCmdBarExists(): Boolean;
    procedure OnBtnClick(const Ctrl: CommandBarButton; var CancelDefault: WordBool);
  
  protected
  
    procedure DoOnConnection(ConnectMode: ext_ConnectMode; const AddInInst: IDispatch;
    	var custom: PSafeArray); override;
    procedure DoOnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); override;
    procedure DoOnAddInsUpdate(var custom: PSafeArray); override;
    procedure DoOnStartupComplete(var custom: PSafeArray); override;
    procedure DoOnBeginShutdown(var custom: PSafeArray); override;

    function BindToHost(): Boolean;
    function UnbindFromHost(): Boolean;

  end;

//******************************************************************************
//******************************************************************************

implementation

uses Windows, ComServ, SysUtils, Controls, MSOAddIn_Defs, Form_AddrInfo_U, MSOAddIn_Utils;


//******************************************************************************
//******************************************************************************

procedure TWordAddIn.DoOnAddInsUpdate(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TWordAddIn.DoOnAddInsUpdate'#10);
  {$ENDIF}
end;

//******************************************************************************
procedure TWordAddIn.DoOnBeginShutdown(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TWordAddIn.DoOnBeginShutdown'#10);
  {$ENDIF}
end;

//******************************************************************************
procedure TWordAddIn.DoOnConnection(ConnectMode: ext_ConnectMode;
  const AddInInst: IDispatch; var custom: PSafeArray);
begin
  Self.BindToHost();
end;

//******************************************************************************
procedure TWordAddIn.DoOnDisconnection(RemoveMode: ext_DisconnectMode;
  var custom: PSafeArray);
begin
  Self.UnbindFromHost();

  // removed by user from COM add-in manager:
  if RemoveMode = ext_dm_UserClosed then
  begin
    Self.FCommandBar.Delete();
  end; // if

end;

//******************************************************************************
procedure TWordAddIn.DoOnStartupComplete(var custom: PSafeArray);
begin
  {$IFDEF DEBUG}
  OutputDebugString('TWordAddIn.DoOnStartupComplete'#10);
  {$ENDIF}
end;

{{*** VK ***********************************************************************
  Function   :BindToHost
  Purpose    :binds addin to host, creates command bar, etc.
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TWordAddIn.BindToHost: Boolean;
begin

	try

    if Self.IsCmdBarExists() then
    begin
      //(VK): find existing button:
      FCommandBarBtn := Self.FCommandBar.FindControl( msoControlButton, 1, WORDADDIN_BTN_NAME + TAG_SUFFIX, 1, 1 )  as CommandBarButton;
    end
    else
    begin

    	//(VK): add command bar:
      Self.FCommandBar := Self.FCommandBars.Add( WORDADDIN_CMD_BAR_NAME, msoBarTop, 0, False );

			//(VK): add button:
      Self.FCommandBarBtn := Self.FCommandBar.Controls.Add( msoControlButton, 1, 0, 1, False ) as CommandBarButton;

      Self.FCommandBarBtn.Set_Tag( WORDADDIN_BTN_NAME + TAG_SUFFIX );
      Self.FCommandBarBtn.Set_TooltipText( WORDADDIN_BTN_NAME );
      Self.FCommandBarBtn.Set_Style( msoButtonCaption );
      Self.FCommandBarBtn.Set_Caption( WORDADDIN_BTN_NAME );

      // user should see newly created command bar:
      Self.FCommandBar.Set_Visible( True );

    end;// if-else

  except
    on E: Exception do
    begin
			{$IFDEF DEBUG}
      OutputDebugString(Pchar(E.Message));
      {$ENDIF}
			Result := False; Exit;
    end;
  end;// try-except


  //(VK): 'connect' button to event sink:
  if Self.FBtnEventSink = nil then
  	Self.FBtnEventSink := TCommandBarButtonEventsSink.Create(DIID__CommandBarButtonEvents);

	Self.FBtnEventSink.OnCommandBarButtonClick := Self.OnBtnClick;

	Result := Self.FBtnEventSink.Connect( Self.FCommandBarBtn );

end;

{{*** VK ***********************************************************************
  Function   :UnbindFromHost
  Purpose    :unbinds addin from host
  Parameters :
  Returns    :true on success
*******************************************************************************}
function TWordAddIn.UnbindFromHost: Boolean;
begin
 	Result := Self.FBtnEventSink.Disconnect( Self.FCommandBarBtn );
  Self.FBtnEventSink := nil;
end;

{{*** VK ***********************************************************************
  Function   :IsCmdBarExists
  Purpose    :checks if named command bar exists
  Parameters :CmdBarName - name of command bar to check
  Returns    :true if named command bar exists
*******************************************************************************}
function TWordAddIn.IsCmdBarExists: Boolean;
var
	BarIndex: Integer;
begin

  BarIndex := MSOAddIn_Utils.FindCommandBar( Self.FCommandBars, WORDADDIN_CMD_BAR_NAME );
  Result := BarIndex > 0;
  if Result then
  begin
    Self.FCommandBar := Self.FCommandBars.Item[BarIndex];
  end; // if

end;

{{*** VK ***********************************************************************
  Procedure  :OnBtnClick
  Purpose    :addin's command button OnClick handler
  Parameters :
*******************************************************************************}
procedure TWordAddIn.OnBtnClick(const Ctrl: CommandBarButton; var CancelDefault: WordBool);
var
	Form_AddrInfo: TForm_AddrInfo;
  TmpStr: WideString;
begin

	Form_AddrInfo := TForm_AddrInfo.Create(nil);
  if Form_AddrInfo.ShowModal = mrOK then
  begin

    TmpStr := 'Name: ' + Form_AddrInfo.Edit_Name.Text +
      #10'Phone: ' + Form_AddrInfo.Edit_Phone.Text +
      #10'E-Mail: ' + Form_AddrInfo.Edit_Email.Text;

  	try

      with Self.FHostApp as WordApplication do
      begin

        if Documents.Count = 0 then
        begin
          Documents.Add( EmptyParam, EmptyParam, EmptyParam, EmptyParam );
        end; // if

        Selection.InsertAfter( TmpStr );
        Selection.Set_Start(0);
        Selection.Set_End_(0);
      end; // with

    except
      on E: Exception do
      begin
        {$IFDEF DEBUG}
        OutputDebugString(PChar(E.Message));
        {$ENDIF}
      end;
    end;// try-except

  end;// if

	Form_AddrInfo.Free;

end;

//******************************************************************************
//******************************************************************************

initialization

  TMSOAddInFactory.Create( WORD_APP, WORDADDIN_NAME,
    WORDADDIN_DESCRIPTION, $03,
    ComServer, TWordAddIn, Class_WordAddIn,
    ciMultiInstance, tmApartment
  );

end.
