unit hfLib;

interface

uses
  {Delphi}
  Windows,
  classes,
  math,
  sysutils;

const
  ALPHA_ONLY                            =
    'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
  NOT_FOUND                             = 0;
  CRLF= #13#10;
  MIN_STRING                            = 0;
  MAX_STRING                            = 4095;
  NULL_TERMINATOR = #0;

  {Define rrString = ShortString (PChar compatible)}
type
  rrString = array[MIN_STRING..MAX_STRING] of Char;

function BreakLineIntoWords(theLine: string): string;  overload;
function BreakLineIntoWords(theLine: string; delims: string): string;  overload;
function BreakLineIntoWords(theLine: string; delims: string; preserveDots: boolean): string;
  overload;
function ConvertStringTorrString(theString: string): rrString;
function CountWords(theLine: string): integer; overload;
function CountWords(theLine: string; delims: string): integer; overload;
function DeleteExtraSpaces(theLine: string): string;
function DeleteThisChar(theLine: string; theChar: string): string;
function EndWithBackslash(inputString: string): string;
function ExtractWord(theLine: string; WordNumber: integer): string; overload;
function ExtractWord(theLine: string; WordNumber: integer; delims: string):
  string; overload;
function ExtractWord(WordNumber: integer; theLine: string; delims: string):
  string; overload;
function GetDataAfter(dataIn: string; matchTo: string; UpTo: string): string;
function GetMachineName: string;
function GetSysDir: string;
function GetTempDir: string;
function GetUniqueTempFilename(fqPathAndFilename: string): string; overload;
function GetUniqueTempFilename(inputPath, inputPrefix, inputDesiredExt:  string): string; overload;
function GetUser: string;
function JustNamePartOfFile(theFileName: string): string;
procedure PrintLines(InText, FQFileName: string);
function StartsWith(inText, what: string): boolean;
function StripNonAlpha(theString: string): string;
procedure TakeSides(inputString: string; inputDivisor: string; var outputLeft, outputRight: string);
function Year(aDateTime: TDateTime): integer;

implementation

function BreakLineIntoWords(theLine: string): string;
begin
  theLine := DeleteExtraSpaces(theLine);
  result := SysUtils.StringReplace(theLine, ' ', CRLF, [rfReplaceAll, rfIgnoreCase]);
  result := SysUtils.StringReplace(result, #13#10#13#10, CRLF, [rfReplaceAll, rfIgnoreCase]);
end;

function BreakLineIntoWords(theLine: string; delims: string): string;
var
  j                                     : integer;
  BreakOnThis                           : string;
begin
  result := DeleteExtraSpaces(theLine);
  for j := 1 to Length(delims) do
  begin
    BreakOnThis := delims[j];
    result := SysUtils.StringReplace(result, BreakOnThis, CRLF, [rfReplaceAll, rfIgnoreCase]);
  end;
  result := SysUtils.StringReplace(result, #13#10#13#10, CRLF, [rfReplaceAll, rfIgnoreCase]);
end;

function BreakLineIntoWords(theLine: string; delims: string; preserveDots: boolean): string;
var
  j                                     : integer;
  BreakOnThis                           : string;
begin
  // PRESERVES DOTS IN PHRASES BUT NOT PERIODS
  if not preserveDots then
  begin
    result := BreakLineIntoWords(theLine, delims);
    exit;
  end;
  result := DeleteExtraSpaces(theLine);
  for j := 1 to Length(delims) do
  begin
    BreakOnThis := delims[j];
    if BreakOnThis <> '.' then
      result := SysUtils.StringReplace(result, BreakOnThis, #13#10, [rfReplaceAll, rfIgnoreCase]);
  end;
  result := SysUtils.StringReplace(result, '. ', CRLF, [rfReplaceAll, rfIgnoreCase]);
end;

function ConvertStringTorrString(theString: string): rrString;
var
  i: integer;
begin
  result := NULL_TERMINATOR;
  if Length(theString) = 0 then
    Exit;
  for i := 0 to Min(MAX_STRING - 1, Length(theString) - 1) do
  begin
    result[i] := theString[i + 1];
  end;
  if (Length(theString) < MAX_STRING) then
    result[Length(theString)] := NULL_TERMINATOR
  else
    result[MAX_STRING] := NULL_TERMINATOR;
end;

function CountWords(theLine: string): integer;
var
  tmp                                   : TStrings;
  i                                     : integer;
begin
  if Trim(theLine) = '' then
    result := 0
  else
  begin
    theLine := DeleteExtraSpaces(theLine);
    tmp := TStringList.Create;
    tmp.Text := BreakLineIntoWords(theLine);
    for i := tmp.Count - 1 downto 0 do
      if Trim(tmp[i]) = '' then
        tmp.Delete(i);
    result := tmp.Count;
    tmp.Free;
  end;
end;

function CountWords(theLine: string; delims: string): integer;
var
  tmp                                   : TStrings;
begin
  if Trim(theLine) = '' then
    result := 0
  else
  begin
    tmp := TStringList.Create;
    tmp.Text := BreakLineIntoWords(theLine, delims);
    result := tmp.Count;
    tmp.Free;
  end;
end;

function DeleteExtraSpaces(theLine: string): string;
begin
  while Pos('  ', theLine) > NOT_FOUND do
    theLine := sysUtils.StringReplace(theLine, '  ', ' ', [rfReplaceAll]);
  result := theLine;
end;

function DeleteThisChar(theLine: string; theChar: string): string;
begin
  result := StringReplace(theLine, theChar, '', [rfReplaceAll])
end;

function EndWithBackslash(inputString: string): string;
begin
  if Trim(inputString) = '' then
    result := inputString
  else
    if inputString[Length(inputString)] <> '\' then
      result := inputString + '\'
    else
      result := inputString;
end;

function ExtractWord(theLine: string; WordNumber: integer): string;
var
  tmp                                   : TStrings;
  i                                     : integer;
begin
  if (Trim(theLine) = '') or (WordNumber = 0) then
    result := ''
  else
  begin
    tmp := TStringList.Create;
    theLine := DeleteExtraSpaces(theLine);
    tmp.Text := BreakLineIntoWords(theLine);
    for i := tmp.Count - 1 downto 0 do
      if Trim(tmp[i]) = '' then
        tmp.Delete(i);
    result := tmp[WordNumber - 1];
    tmp.Free;
  end;
end;

function ExtractWord(theLine: string; WordNumber: integer; delims: string):
  string;
var
  tmp                                   : TStrings;
begin
  if (Trim(theLine) = '') or (WordNumber <= 0) then
    result := ''
  else
  begin
    if Length(delims) = 0 then
      delims := ' ';
    tmp := TStringList.Create;
    tmp.Text := BreakLineIntoWords(theLine, delims);
    if tmp.Count >= WordNumber then
      result := tmp[WordNumber - 1]
    else
      result := '';
    tmp.Free;
  end;
end;

function ExtractWord(WordNumber: integer; theLine: string; delims: string):
  string;
var
  tmp                                   : TStrings;
begin
  if (Trim(theLine) = '') or (WordNumber = 0) then
    result := ''
  else
  begin
    if Length(delims) = 0 then
      delims := ' ';
    tmp := TStringList.Create;
    tmp.Text := BreakLineIntoWords(theLine, delims);
    if tmp.Count >= WordNumber then
      result := tmp[WordNumber - 1]
    else
      result := '';
    tmp.Free;
  end;
end;

function GetDataAfter(dataIn: string; matchTo: string; UpTo: string): string;
var
  startPos: integer;
  endPos: integer;
  //left,
  right: string;
begin
  result := '';
  startPos := Pos(matchTo, dataIn);
  if startPos > 0 then
  begin
    //    left := Copy(dataIn, 1, startPos - 1);
    right := Copy(dataIn, startPos + Length(matchTo), Length(dataIn));
    endPos := Pos(UpTo, right);
    if endPos > 0 then
      result := Copy(dataIn, startPos + Length(matchTo), endPos - 1)
    else
      result := right;
  end;
end;

function GetMachineName: string;
var
  uName: array[0..511] of char;
  aSize: DWORD;
begin
  uName := #0;
  aSize := 512;
  Windows.GetComputerName(uName, aSize);
  result := Trim(result + ' on ' + uName);
end;

function GetSysDir: string;
var
  SDir: array[0..511] of char;
begin
  SDir := #0;
  Windows.GetSystemDirectory(SDir, 511);
  result := SDir;
end;

function GetTempDir: string;
var
  TDir: array[0..511] of char;
begin
  TDir := #0;
  Windows.GetTempPath(511, TDir);
  result := sysutils.IncludeTrailingBackslash(TDir);
end;

function GetUniqueTempFilename(inputPath, inputPrefix, inputDesiredExt:  string): string;
const
  USE_NOW = 0;
  //  Let Windows gen a random number from current date time
var
  thePath: rrString;
  theOutFile: rrString;
  thePrefix: rrString;
begin
  theOutFile := #0;
  thePath := ConvertStringTorrString(inputPath);
  thePrefix := ConvertStringTorrString(inputPrefix);
  Windows.GetTempFileName(thePath, thePrefix, USE_NOW, theOutFile);
  result := ChangeFileExt(theOutfile, inputDesiredExt);
end;

function GetUniqueTempFilename(fqPathAndFilename: string): string;
const
  ENSURE_UNIQUE_AND_USE_NOW = 0;
  //  Let Windows gen a random number from current date time
var
  thePath: rrString;
  theOutFile: rrString;
  thePrefix: rrString;
  inputDesiredExt: rrString;
begin
  inputDesiredExt :=
    ConvertStringTorrString(ExtractFileExt(fqPathAndFilename));
  thePath := ConvertStringTorrString(ExtractFilePath(fqPathAndFilename));
  thePreFix :=
    ConvertStringTorrString(JustnamePartofFile(fqPathAndFilename));
  theOutFile := #0;
  Windows.GetTempFileName(thePath, thePrefix, ENSURE_UNIQUE_AND_USE_NOW,
    theOutFile);
  DeleteFile(theOutfile);
  result := ChangeFileExt(theOutfile, inputDesiredExt);
end;

function GetUser: string;
var
  UserName: array[0..511] of char;
  i: DWORD;
begin
  result := '';
  i := 512;
  UserName := #0;
  try
    if GetUserName(UserName, i) then
      result := UserName;
  except
  end;
  if Trim(UserName) = '' then
    result := GetMachineName;
end;

function JustNamePartOfFile(theFileName: string): string;
var
  i, lngth, FilenameStartsAt, extStartsAt: integer;
begin
  result := theFileName;
  lngth := Length(theFileName);
  extStartsAt := lngth;
  for i := lngth downto 1 do
  begin
    if theFileName[i] = '.' then
    begin
      extStartsAt := i - 1;
      Break;
    end;
  end;

  result := Copy(theFileName, 1, extStartsAt);
  FilenameStartsAt := 1;
  lngth := Length(result);
  for i := lngth downto 1 do
  begin
    if theFileName[i] in ['\', ':'] then
    begin
      FilenameStartsAt := i + 1;
      Break;
    end;
  end;

  result := Copy(theFileName, FilenameStartsAt, extStartsAt - FilenameStartsAt
    + 1);
end;

procedure PrintLines(InText, FQFileName: string);
var
  print                                 : TStringList;
begin
  print := TStringList.Create;
  print.Text := InText;
  print.SaveToFile(FQFilename);

  print.Free;
end;

function StartsWith(inText, what: string): boolean;
//  case sensitive!
begin
  if Pos(what, inText) = 1 then
    result := true
  else
    result := false;
end;

function StripNonAlpha(theString: string): string;
var
  i                                     : integer;
begin
  for i := 1 to Length(theString) do
  begin
    if Pos(theString[i], ALPHA_ONLY) > NOT_FOUND then
      result := result + theString[i];
  end;
end;

procedure TakeSides(inputString: string; inputDivisor: string; var outputLeft, outputRight: string);
var
  p: integer;
begin
  outputLeft := inputString;
  outputRight := '';
  p := Pos(inputDivisor, inputString);
  if p > 0 then
  begin
    outputLeft := Copy(inputString, 1, p - 1);
    if Length(inputString) > p then
      outputRight := Copy(inputString, p + Length(inputDivisor), Length(inputString) - p - Length(inputDivisor) + 1);
  end;
end;

function Year(aDateTime: TDateTime): integer;
var
  mo, day, yr                           : word;
begin
  result := 0;
  if (not (aDateTime >= 0)) then
    Exit;
  DecodeDate(aDateTime, yr, mo, day);
  result := yr;
end;

end.
