//rev 1.01
unit THostChecker_;

interface
uses
 Forms,Dialogs, Windows,Controls, Sysutils,Classes,shProgs,PingThrd, ExtCtrls;


type
  THostChecker=class;

  THostChecker_PingMode= (hpmSimultaneous,hpmOneByOne);
  THostChecker_OnCheckLink=procedure (Sender:TObject;const LinkExist:boolean) of object;

  THostChecker=class(TComponent)
   private
    aTimer:TTimer;

    fLinkRespCnt,fLinkFailCnt:Integer;
    fAutoCheck: boolean;
    fHostName: string;
    fOnCheckLink: THostChecker_OnCheckLink;
    fOnNoLink: TNotifyEvent;
    fOnLink: TNotifyEvent;
    fCheckPeriod: word;

    fTTL: byte;
    fTTLEx: byte;

    fPingCount: byte;
    fPingCountEx: byte;

    fTimeOut: word;
    fTimeOutEx: word;

    fPingSize: byte;
    fPingSizeEx: byte;

    FPMode: THostChecker_PingMode;
    FPModeEx: THostChecker_PingMode;

    FOnLog: TOnLog;



    FID: string;


    procedure setAutoCheck(const Value: boolean);
    procedure setCheckPeriod(const Value: word);
    procedure log(Msg:string;LogSt:TLogStatus; LogL:byte);
    procedure OnTimer(Sender:TObject);
    procedure AfterCheckLink(Sender:TObject);
    procedure setPingCount(const Value: byte);
    procedure setHostName(const Value: string);
    procedure setPingSize(const Value: byte);
    procedure setTimeOut(const Value: word);
    procedure setTTL(const Value: byte);
   public
    constructor create(AOwner:TComponent);override;
    destructor  destroy;override;
    procedure CheckLink(Host:string; PingCnt:byte;PSize:integer = 1; PngMode:THostChecker_PingMode = hpmSimultaneous; TOut:integer = 1500;  pTTL:integer = 64 );
    procedure Execute;
   published
    property LogID:string read FID write FID;
    property AutoCheck:boolean read fAutoCheck write setAutoCheck;
    property AutoCheckPeriod:word read fCheckPeriod write setCheckPeriod;
    property PingCount:byte read fPingCount write  setPingCount;
    property PingSize:byte read fPingSize write setPingSize;
    property TTL:byte read fTTL write setTTL;
    property PingTimeOut:word read fTimeOut write setTimeOut;
    property HostName:string read fHostName write setHostName;
    property OnLink:TNotifyEvent read fOnLink write fOnLink;
    property OnNoLink:TNotifyEvent read fOnNoLink write fOnNoLink;
    property OnCheckLink:THostChecker_OnCheckLink read fOnCheckLink write fOnCheckLink;
    property OnLog:TOnLog read FOnLog write FOnLog;
    property PingMode:THostChecker_PingMode read FPMode write FPMode;
  end;

 procedure Register;



implementation


procedure register;
begin
  registercomponents('Zeta',[THostChecker]);
end;

procedure THostChecker.AfterCheckLink(Sender: TObject);
const
    response1 = 'Ping for %s, %s' ;
    response2 = 'Ping for %s, received %d bytes from %s in %dms' ;
begin
    if Application.Terminated then exit ;
    inc(FLinkRespCnt);
 // this event is thread safe, all publics from the thread are available here
    with Sender as TPingThread do
        if ReplyTotal <> 0 then begin
                Log(Format (response2, [ PingHostName,
                         ReplyDataSize, ReplyIPAddr, ReplyRTT]),stDontCare,llFile);
        end
        else begin
         Log(Format (response1, [ PingHostName, ErrString]),stFailed,llFile) ;
         inc(FLinkFailCnt);
        end;
    if FLinkRespCnt=fPingCountEx then begin
      if FLinkFailCnt<fPingCountEx then begin //there is link
          log('There is link to '+ fHostName+'.',stReady,llList);
          if  Assigned(fOnLink) then fOnLink(self);
          if  Assigned(fOnCheckLink) then fOnCheckLink(self,true);
         end else begin                     //sorry no link
          log('No link to '+ fHostName+'.',stReady,llList);
          if  Assigned(fOnNoLink) then fOnNoLink(self);
          if  Assigned(fOnCheckLink) then fOnCheckLink(self,false);
       end;
       FLinkFailCnt:=0;
       FLinkRespCnt:=0;
  end
   else if FPModeEx=hpmOneByOne
     then CheckLink(fHostName,fPingCountEx,fPingSizeEx,FPModeEx,fTTLEx,fTTLEx);


end;

procedure THostChecker.CheckLink(Host:string; PingCnt:byte;PSize:integer = 1;PngMode:THostChecker_PingMode = hpmSimultaneous;
   TOut:integer = 1500;  pTTL:integer = 64 );
 var i:integer;
   cnt:integer;
begin

 //  THostChecker_PingMode= (hpmSimultaneous,hpmOneByOne);

 FPModeEx:=PngMode;
 fPingCountEx:=PingCnt;
 fPingSizeEx:=PingSize;
 fTimeOutEx:=TOut;
 fTTLEx:=TTL;

 if FPModeEx=hpmSimultaneous then begin
  FLinkFailCnt:=0;
  FLinkRespCnt:=0;
  cnt:=fPingCount
 end
  else cnt:=1;
     for i:=0 to cnt-1 do
      with TPingThread.Create (True) do   // create suspended
            begin
                PingAddThread (ThreadId) ;      // keep threadid so it's freed
                FreeOnTerminate := True;
                OnTerminate := AfterCheckLink;    // where we get the response
                PingSize:=fPingSizeEx;
                PingHostName := Host ;  // host name or IP address to ping
                PingTimeout := fTimeOutEx;           // ms
                PingTTL := fTTLEx;                 // hops
                PingLookupReply := false ;      // don't need response host name lookup
                Resume ;                        // start it now
             end ;

end;

constructor THostChecker.create(AOwner: TComponent);
begin
  inherited create(AOwner);
   aTimer:=TTimer.Create(self);
   aTimer.Enabled:=false;
   aTimer.OnTimer:=OnTimer;
   fAutoCheck:=false;
   fCheckPeriod:=10000;
   FID:='Link checker';
   fTTL:=64;
   fPingSize:=1;
   fTimeOut:=1500;
   fPingCount:=5;
   fHostName:='localhost';
   FLinkFailCnt:=0;
   FLinkRespCnt:=0;
 end;



destructor THostChecker.destroy;
begin
  if assigned(aTimer) then aTimer.Free;
  inherited;
end;

procedure THostChecker.Execute;
begin
 CheckLink(fHostName,fPingCount,fPingSize,FPMode,fTimeOut,fTTL);
end;

procedure THostChecker.log(Msg: string; LogSt: TLogStatus; LogL: byte);
begin
 if assigned(FOnLog) then
 FOnLog(self,FId,Msg,LogL,LogSt);
end;

procedure THostChecker.OnTimer(Sender: TObject);
begin
if Application.Terminated then exit;
if not fAutoCheck then exit;
 Execute;
end;

procedure THostChecker.setAutoCheck(const Value: boolean);
begin
  aTimer.Enabled:=Value;
  fAutoCheck := Value;
end;

procedure THostChecker.setCheckPeriod(const Value: word);
begin
 if Value=0 then exit;
  aTimer.Interval:=Value;
  fCheckPeriod := Value;

end;

procedure THostChecker.setHostName(const Value: string);
begin
if length(trim(fHostName))>0 then
  fHostName := Value;
end;

procedure THostChecker.setPingCount(const Value: byte);
begin
if Value>0 then
  fPingCount := Value;
end;

procedure THostChecker.setPingSize(const Value: byte);
begin
if Value>0 then
  fPingSize := Value;
end;

procedure THostChecker.setTimeOut(const Value: word);
begin
 if Value>0 then
  fTimeOut := Value;
end;

procedure THostChecker.setTTL(const Value: byte);
begin
 if Value>0 then
  fTTL := Value;
end;

end.
