//==============================================================================
//    PBDbIbx.pas - CGI Framework module. Implements TDBIbxTagInterpreter
//                  component.
//    Copyright (C) 2002-2003  Ivan Camilo da Cruz
//==============================================================================
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//==============================================================================
//    Este programa  software livre; voc pode redistribu-lo e/ou
//    modific-lo sob os termos da Licena Pblica Geral GNU, conforme
//    publicada pela Free Software Foundation; tanto a verso 2 da
//    Licena como (a seu critrio) qualquer verso mais nova.
//
//    Este programa  distribudo na expectativa de ser til, mas SEM
//    QUALQUER GARANTIA; sem mesmo a garantia implcita de
//    COMERCIALIZAO ou de ADEQUAO A QUALQUER PROPSITO EM
//    PARTICULAR. Consulte a Licena Pblica Geral GNU para obter mais
//    detalhes.
//
//    Voc deve ter recebido uma cpia da Licena Pblica Geral GNU
//    junto com este programa; se no, escreva para a Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//    02111-1307, USA.
//==============================================================================

unit PBDbIbx;

interface

uses SysUtils, Classes, DB, IB, IBDatabase, IBQuery, PageBuilder, PBDb;

type TPBIbxRepositoryItem = class ( TPBDbRepositoryItem );

type TPBIbxRepository = class ( TPBDbRepository )
         protected
             function DoCreateRepositoryItem ( const pName, pSql: String; const pMaster: TDataSource ): TPBDbRepositoryItem; override;
     end;

type TDBIbxTagInterpreter = class ( TDBTagInterpreter )
         private
             fDatabase: TIBDatabase;
             fTransaction: TIBTransaction;
         protected
             function DoCreateRepository: TPBDbRepository; override;
             function DoCreateDataset ( const pSql: String; const pMaster: TDataSource ): TDataSet; override;
         public
             constructor Create ( pOwner: TComponent ); override;
         published
             property Database: TIBDatabase read fDatabase write fDatabase;
             property Transaction: TIBTransaction read fTransaction write fTransaction;
     end;

procedure Register;

implementation

procedure Register;
  begin
    RegisterComponents ( 'CGI Framework', [ TDBIbxTagInterpreter ] );
  end;

{ TPBIbxRepository }

function TPBIbxRepository.DoCreateRepositoryItem ( const pName: String; const pSql: String; const pMaster: TDataSource ): TPBDbRepositoryItem;
  begin
    Result := TPBIbxRepositoryItem.Create ( pName, pSql, TagInterpreter, pMaster );
  end;

{ TDBIbxTagInterpreter }

constructor TDBIbxTagInterpreter.Create ( pOwner: TComponent );
  begin
    inherited;

    fMasterDataSource := nil;
  end;

function TDBIbxTagInterpreter.DoCreateRepository: TPBDbRepository;
  begin
    Result := TPBIbxRepository.Create ( Self );
  end;

function TDBIbxTagInterpreter.DoCreateDataset ( const pSql: String; const pMaster: TDataSource ): TDataSet;
  begin
    Result := TIBQuery.Create ( nil );
    TIBQuery ( Result ).Database := Database;
    TIBQuery ( Result ).Transaction := Transaction;
    TIBQuery ( Result ).SQL.Text := pSql;
    TIBQuery ( Result ).DataSource := pMaster;
  end;

end.
