{
     2000-2003  Sovren Group, Inc.  May be used if this header
                 is retained and any code ,ods are noted.

    OWNER:              Sovren Group, Inc.     www.sovren.com
    AUTHOR:             Robert H. Ruff         info@sovren.com
    DELPHI VERSION:     7.  Should be backward compatible to all 32-bit.
    CUSTOM UNITS USED:
    DESCRIPTION:        prForm allows a form's size and position to be persisted.
                        In addition, it automatically persists the values contained
                        in other GUI components like TEdits (with an "opt-out"
                        feature).
    MODIFIED:

    Differs from prSharedForm in that its settings are not persisted to the
            application directory, but rather, to the local user's
            Windows directory.


    YOU DO NOT NEED TO USE THIS LIKE AN ACTUAL GUI COMPONENT!
    To change any form to a prForm, just add "prForm" to the
    "uses" section in the interface, and change the form
    declaration to be of class TprForm.  That's it.

    TmyFormNameWhatever = class(TprForm)
}
unit prForm;

interface

uses
  Windows,
  Dialogs,
  Classes,
  Forms,
  SysUtils,
  filectrl,
  stdctrls,
  CheckLst,
  Math,
  hfLib,
  Mask;

type
  TprForm = class(TForm)
  private
    FINIholder: TStrings;
    procedure SetINIholder(const Value: TStrings);
    property INIholder: TStrings read FINIholder write SetINIholder;
    function GetControlText(aControlName: string): string;
  protected
  public
  published
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetTempPath: string; virtual;
  end;

procedure Register;

implementation

const
  MAX_ITEMS_PER_CONTROL_TO_SAVE         = 50;

procedure Register;
begin
  RegisterComponents('Standard', [TprForm]);
end;

{ TprForm }

constructor TprForm.Create(AOwner: TComponent);
var
  i                                     : integer;
  WinState, leftSide, rightSide         : string;
  anObject                              : TObject;
begin
  inherited;
  INIholder := TStringList.Create;
  try
    if FileExists(GetTempPath + Name + '.INI') then
    begin
      INIholder.LoadFromFile(GetTempPath + Name + '.INI');
    end
    else
      Exit;

    for i := 0 to ComponentCount - 1 do
    begin
      //  Sometimes we want to retain design-time info, so allow opt-out using Tag property
      if Components[i].Tag = 1234567 then Continue;
      if Components[i] is TMemo then
      begin
        anObject := Components[i];
        begin
          if FileExists(GetTempPath + (anObject as TMemo).Name + '.dat') then
            (anObject as TMemo).Lines.LoadFromFile(GetTempPath + (anObject as
              TMemo).Name + '.dat');
        end;
      end;

      if Components[i] is TEdit then
      begin
        anObject := Components[i];
        begin
          (anObject as TEdit).Text := GetControlText((anObject as TEdit).Name);
        end;
      end;

      if Components[i] is TMaskEdit then
      begin
        anObject := Components[i];
        begin
          (anObject as TMaskEdit).Text := GetControlText((anObject as TMaskEdit).Name);
        end;
      end;

      if Components[i] is TListBox then
      begin
        anObject := Components[i];
        rightSide := GetControlText((anObject as TListBox).Name);
        while Trim(rightSide) <> '' do
        begin
          hfLib.TakeSides(rightSide, '^', leftSide, rightSide);
          if Trim(leftSide) <> '' then
            (anObject as TListBox).Items.Add(leftSide);
        end;
      end;

      if Components[i] is TCheckListBox then
      begin
        anObject := Components[i];
        rightSide := GetControlText((anObject as TCheckListBox).Name);
        while Trim(rightSide) <> '' do
        begin
          hfLib.TakeSides(rightSide, '^', leftSide, rightSide);
          if Trim(leftSide) <> '' then
            (anObject as TCheckListBox).Items.Add(leftSide);
        end;
      end;

      if Components[i] is TComboBox then
      begin
        anObject := Components[i];
        rightSide := GetControlText((anObject as TComboBox).Name);
        while Trim(rightSide) <> '' do
        begin
          hfLib.TakeSides(rightSide, '^', leftSide, rightSide);
          if Trim(leftSide) <> '' then
            if (anObject as TComboBox).Items.IndexOf(leftSide) < 0 then
              (anObject as TComboBox).Items.Add(leftSide);
        end;
      end;
    end;

    //  Load window coordinates
    if GetControlText('.Left') <> '' then
      Left := StrToInt(GetControlText('.Left'));
    if GetControlText('.Top') <> '' then
      Top := StrToInt(GetControlText('.Top'));
    if GetControlText('.Width') <> '' then
      Width := StrToInt(GetControlText('.Width'));
    if GetControlText('.Height') <> '' then
      Height := StrToInt(GetControlText('.Height'));

    //  Load window state
    winState := GetControlText('.WindowState');
    if winState = 'NORMAL' then
      WindowState := wsNormal
    else
      if winState = 'MINIMIZED' then
        WindowState := wsMinimized
      else
        if winState = 'MAXIMIZED' then
          WindowState := wsMaximized;
  finally
    INIholder.Free;
  end;
end;

destructor TprForm.Destroy;
var
  i, j                                  : integer;
  WinState, SaveAs, tmpStr              : string;
  anObject                              : TObject;
begin
  INIholder := TStringList.Create;
  try
    for i := 0 to ComponentCount - 1 do
    begin
      tmpStr := '';
      if Components[i] is TEdit then
      begin
        anObject := Components[i];
        begin
          INIholder.Add((anObject as TEdit).Name + '=' + (anObject as TEdit).Text);
        end;
      end;

      tmpStr := '';
      if Components[i] is TMaskEdit then
      begin
        anObject := Components[i];
        begin
          INIholder.Add((anObject as TMaskEdit).Name + '=' + (anObject as TMaskEdit).Text);
        end;
      end;

      tmpStr := '';
      if Components[i] is TMemo then
      begin
        anObject := Components[i];
        begin
          (anObject as TMemo).Lines.SaveToFile(GetTempPath + (anObject as TMemo).Name + '.dat');
        end;
      end;

      if Components[i] is TListBox then
      begin
        anObject := Components[i];
        tmpStr := '';
        for j := 0 to Min(MAX_ITEMS_PER_CONTROL_TO_SAVE, (anObject as TListBox).Items.Count - 1) do
        begin
          tmpStr := tmpStr + '^' + (anObject as TListBox).Items[j];
        end;
        INIholder.Add((anObject as TListBox).Name + '=' + tmpStr);
      end;
      tmpStr := '';
      if Components[i] is TCheckListBox then
      begin
        anObject := Components[i];
        for j := 0 to Min(MAX_ITEMS_PER_CONTROL_TO_SAVE, (anObject as TCheckListBox).Items.Count - 1)
          do
        begin
          tmpStr := tmpStr + '^' + (anObject as TCheckListBox).Items[j];
        end;
        INIholder.Add((anObject as TCheckListBox).Name + '=' + tmpStr);
      end;
      tmpStr := '';
      if Components[i] is TComboBox then
      begin
        anObject := Components[i];
        for j := 0 to Min(MAX_ITEMS_PER_CONTROL_TO_SAVE, (anObject as TComboBox).Items.Count - 1) do
        begin
          tmpStr := tmpStr + '^' + (anObject as TComboBox).Items[j];
        end;
        INIholder.Add((anObject as TComboBox).Name + '=' + tmpStr);
      end;
      tmpStr := '';
    end;

    //  Save window state
    if WindowState = wsNormal then
      winState := 'NORMAL'
    else
      if WindowState = wsMinimized then
        winState := 'MINIMIZED'
      else
        if WindowState = wsMaximized then
          winState := 'MAXIMIZED';

    //  Save window coordinates
    INIholder.Add('.WindowState=' + winState);

    if WindowState = wsMaximized then
      INIholder.Add('.Left=' + IntToStr(Round(0.8 * Left)))
    else
      INIholder.Add('.Left=' + IntToStr(Left));
    if WindowState = wsMaximized then
      INIholder.Add('.Top=' + IntToStr(Round(0.8 * Top)))
    else
      INIholder.Add('.Top=' + IntToStr(Top));
    if WindowState = wsMaximized then
      INIholder.Add('.Width=' + IntToStr(Round(0.8 * Width)))
    else
      INIholder.Add('.Width=' + IntToStr(Width));
    if WindowState = wsMaximized then
      INIholder.Add('.Height=' + IntToStr(Round(0.8 * Height)))
    else
      INIholder.Add('.Height=' + IntToStr(Height));

    try
      SaveAs := GetTempPath + Name + '.INI';
      INIholder.SaveToFile(SaveAs);
    finally
      INIholder.Free;
    end;
  finally
    try
      inherited;
    except
    end;
  end;
end;

function TprForm.GetControlText(aControlName: string): string;
begin
  result := INIholder.Values[aControlName];
end;

procedure TprForm.SetINIholder(const Value: TStrings);
begin
  FINIholder := Value;
end;

class function TprForm.GetTempPath: string;
var
  user                                  : string;
begin
  try
    user := Trim(hfLib.GetUser);
    if user = '' then
    begin
      user := hfLib.GetMachineName;
    end;
    user := user + '-';
    result := hfLib.EndWithBackslash(ExtractFilePath(Application.ExeName)) + user;
  except
    try
      result := hfLib.EndWithBackslash(ExtractFilePath(Application.ExeName));
    except
      result := 'prFormError-';
    end;
  end;
end;

end.

