//==============================================================================
//    dmmain.pas - Module of staticgen tool. Implements program functions.
//                 staticgen is part of CGI Framework package.
//    Copyright (C) 2002-2003  Ivan Camilo da Cruz
//==============================================================================
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//==============================================================================
//    Este programa  software livre; voc pode redistribu-lo e/ou
//    modific-lo sob os termos da Licena Pblica Geral GNU, conforme
//    publicada pela Free Software Foundation; tanto a verso 2 da
//    Licena como (a seu critrio) qualquer verso mais nova.
//
//    Este programa  distribudo na expectativa de ser til, mas SEM
//    QUALQUER GARANTIA; sem mesmo a garantia implcita de
//    COMERCIALIZAO ou de ADEQUAO A QUALQUER PROPSITO EM
//    PARTICULAR. Consulte a Licena Pblica Geral GNU para obter mais
//    detalhes.
//
//    Voc deve ter recebido uma cpia da Licena Pblica Geral GNU
//    junto com este programa; se no, escreva para a Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//    02111-1307, USA.
//==============================================================================

unit dmmain;

interface

uses
  SysUtils, Classes, IBDatabase, DB, PageBuilder, PBTools, PBDb, PBDbIbx;

type
  TDtmMain = class(TDataModule)
    IBDb: TIBDatabase;
    IBTransaction: TIBTransaction;
    PbdMain: TPageBuilder;
    PBTagInt: TPBTagInterpreter;
    DBIbxTagInt: TDBIbxTagInterpreter;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
    targetFile: String;
  public
    { Public declarations }
    procedure Process;
  end;

var
  DtmMain: TDtmMain;

implementation

{$R *.dfm}

{ TDtmMain }

procedure TDtmMain.DataModuleCreate ( Sender: TObject );

    procedure showUsage;
      begin
        writeln ( 'Usage: staticgen sourceTemplate [ targetFile ] [ -t:templateDir ] [ -d:interbaseDatabase ]' );
      end;

    function getOption ( option: String ): String;
      begin
        if option [ 3 ] <> ':' then
            showUsage
        else
            Result := Copy ( option, 4, MaxInt );
      end;

    procedure checkOption ( option: String );
      begin
        if Length ( option ) < 4 then
            showUsage
        else if option [ 1 ] <> '-' then
            showUsage
        else if option [ 2 ] = 't' then
            PBTagInt.IncludeDir := getOption ( option )
        else if option [ 2 ] = 'd' then
            IBDb.DatabaseName := getOption ( option )
        else
            showUsage;
      end;

    var i: Integer;

  begin
    IBDb.Close;

    if ParamCount > 0 then
        PbdMain.HTMLFile := ParamStr ( 1 )
    else
        showUsage;

    if ParamCount > 1 then
        if ParamStr ( 2 )[ 1 ] = '-' then
            checkOption ( ParamStr ( 2 ) )
        else
            targetFile := ParamStr ( 2 );

    if targetFile = '' then
        targetFile := ChangeFileExt ( PbdMain.HTMLFile, '.html' );

    for i := 3 to ParamCount do
        checkOption ( ParamStr ( i ) );

    if PBTagInt.IncludeDir = '' then
        PBTagInt.IncludeDir := ExtractFilePath ( PbdMain.HTMLFile );

    if PBTagInt.IncludeDir = '' then
        PBTagInt.IncludeDir := GetCurrentDir;

    if IBDb.DatabaseName <> '' then
      begin
        IBDb.Open;
        IBTransaction.StartTransaction;
      end;
  end;

procedure TDtmMain.DataModuleDestroy ( Sender: TObject );
  begin
    if IBTransaction.Active then
        IBTransaction.Commit;
  end;

procedure TDtmMain.Process;
  var result: Integer;
      content: String;
  begin
    DeleteFile ( targetFile );
    result := FileCreate ( targetFile );
    try
        content := PbdMain.Content;
        FileWrite ( result, content[1], Length ( content ) );
    finally
        FileClose ( result );
    end;
  end;

end.
