//==============================================================================
//    Main.pas - Main form of TestTool. Implements main program functions.
//               TestTool is part of CGI Framework package.
//    Copyright (C) 2002-2003  Ivan Camilo da Cruz
//==============================================================================
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//==============================================================================
//    Este programa  software livre; voc pode redistribu-lo e/ou
//    modific-lo sob os termos da Licena Pblica Geral GNU, conforme
//    publicada pela Free Software Foundation; tanto a verso 2 da
//    Licena como (a seu critrio) qualquer verso mais nova.
//
//    Este programa  distribudo na expectativa de ser til, mas SEM
//    QUALQUER GARANTIA; sem mesmo a garantia implcita de
//    COMERCIALIZAO ou de ADEQUAO A QUALQUER PROPSITO EM
//    PARTICULAR. Consulte a Licena Pblica Geral GNU para obter mais
//    detalhes.
//
//    Voc deve ter recebido uma cpia da Licena Pblica Geral GNU
//    junto com este programa; se no, escreva para a Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//    02111-1307, USA.
//==============================================================================

unit Main;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls, QComCtrls, QButtons, IniFiles, IBDatabase, DB,
  QMenus, QTypes, PageBuilder, PBTools, PBDb, PBDbIbx;

type
  TFrmMain = class(TForm)
    PageBuilder: TPageBuilder;
    PnlBotoes: TPanel;
    BtnOpen: TSpeedButton;
    BtnGenerate: TSpeedButton;
    OpenDialog: TOpenDialog;
    PgcBaseFiles: TPageControl;
    IBDb: TIBDatabase;
    IBTransaction: TIBTransaction;
    BtnExit: TSpeedButton;
    PpmBaseFiles: TPopupMenu;
    MnuCloseTab: TMenuItem;
    Splitter1: TSplitter;
    PBTagInt: TPBTagInterpreter;
    LblTime: TLabel;
    DBIbxTagInt: TDBIbxTagInterpreter;
    MainMenu: TMainMenu;
    MnuFile: TMenuItem;
    MnuOpen: TMenuItem;
    MnuNew: TMenuItem;
    MnuClose: TMenuItem;
    N1: TMenuItem;
    MnuExit: TMenuItem;
    MnuGenerate: TMenuItem;
    MnuAbout: TMenuItem;
    SaveDialog: TSaveDialog;
    N2: TMenuItem;
    MnuOptions: TMenuItem;
    MnuTemplateFont: TMenuItem;
    MnuGeneratedFont: TMenuItem;
    FontDialog: TFontDialog;
    PnlGenerated: TPanel;
    MemoGeneratedHtml: TMemo;
    MnuPreview: TMenuItem;
    BtnPreview: TSpeedButton;
    N3: TMenuItem;
    MnuConfig: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MnuCloseTabClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure MnuNewClick(Sender: TObject);
    procedure MnuOpenClick(Sender: TObject);
    procedure MnuCloseClick(Sender: TObject);
    procedure MnuExitClick(Sender: TObject);
    procedure MnuGenerateClick(Sender: TObject);
    procedure MnuAboutClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure MnuTemplateFontClick(Sender: TObject);
    procedure MnuGeneratedFontClick(Sender: TObject);
    procedure MnuPreviewClick(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MnuConfigClick(Sender: TObject);
  private
    { Private declarations }
    fileNames: TStringList;
    fileMemos: TList;
    templateFont: TFont;
    keepAspect: Boolean;
    aspect: Double;
    function newPage ( tabCaption: String ): TMemo;
    procedure loadFile ( fileName: String );
    procedure saveFile ( idx: Integer );
    function saveAll: Boolean;
  public
    { Public declarations }
  end;

var
  FrmMain: TFrmMain;

implementation

uses About, Config, Globals, Platform;

{$R *.xfm}

const noName: String = 'Noname';    // When changing noName, change also the next const.
      tNoname: Integer = 6;         // Length ( noName )

function strToStyle ( str: String ): TFontStyles;
  var i: Integer;
  begin
    Result := [];
    for i := 1 to Length ( str ) do
        case str[i] of
            'b', 'B': Result := Result + [ fsBold ];
            'i', 'I': Result := Result + [ fsItalic ];
            'u', 'U': Result := Result + [ fsUnderline ];
            's', 'S': Result := Result + [ fsStrikeOut ];
        end;
  end;

function styleToStr ( style: TFontStyles ): String;
  begin
    Result := '';

    if fsBold in style then
        Result := Result + 'B';

    if fsItalic in style then
        Result := Result + 'I';

    if fsUnderline in style then
        Result := Result + 'U';

    if fsStrikeOut in style then
        Result := Result + 'S';
  end;

procedure TFrmMain.FormCreate ( Sender: TObject );
  begin
    keepAspect := true;
    aspect := PgcBaseFiles.Height / ( PgcBaseFiles.Height + PnlGenerated.Height );

    ConfigIni := TMemIniFile.Create ( ChangeFileExt ( Application.ExeName, '.ini' ) );

    fileNames := TStringList.Create;
    fileMemos := TList.Create;

    templateFont := TFont.Create;
    templateFont.Name := ConfigIni.ReadString ( 'Template.Font', 'Name', Font.Name );
    templateFont.Height := ConfigIni.ReadInteger ( 'Template.Font', 'Height', Font.Height );
    templateFont.Style := strToStyle ( ConfigIni.ReadString ( 'Template.Font', 'Style', styleToStr ( Font.Style ) ) );
    templateFont.Color := ConfigIni.ReadInteger ( 'Template.Font', 'Color', Font.Color );

    MemoGeneratedHtml.Font.Name := ConfigIni.ReadString ( 'Generated.Font', 'Name', Font.Name );
    MemoGeneratedHtml.Font.Height := ConfigIni.ReadInteger ( 'Generated.Font', 'Height', Font.Height );
    MemoGeneratedHtml.Font.Style := strToStyle ( ConfigIni.ReadString ( 'Generated.Font', 'Style', styleToStr ( Font.Style ) ) );
    MemoGeneratedHtml.Font.Color := ConfigIni.ReadInteger ( 'Generated.Font', 'Color', Font.Color );
  end;

procedure TFrmMain.FormDestroy ( Sender: TObject );
  begin
    ConfigIni.WriteString  ( 'Template.Font', 'Name',  templateFont.Name );
    ConfigIni.WriteInteger ( 'Template.Font', 'Height',  templateFont.Height );
    ConfigIni.WriteString  ( 'Template.Font', 'Style', styleToStr ( templateFont.Style ) );
    ConfigIni.WriteInteger ( 'Template.Font', 'Color', templateFont.Color );

    ConfigIni.WriteString  ( 'Generated.Font', 'Name',  MemoGeneratedHtml.Font.Name );
    ConfigIni.WriteInteger ( 'Generated.Font', 'Height',  MemoGeneratedHtml.Font.Height );
    ConfigIni.WriteString  ( 'Generated.Font', 'Style', styleToStr ( MemoGeneratedHtml.Font.Style ) );
    ConfigIni.WriteInteger ( 'Generated.Font', 'Color', MemoGeneratedHtml.Font.Color );

    ConfigIni.UpdateFile;

    ConfigIni.Free;
    fileNames.Free;
    fileMemos.Free;
    templateFont.Free;
  end;

procedure TFrmMain.FormShow ( Sender: TObject );
  var arquivos: TStringList;
      i: Integer;
  begin
    arquivos := TStringList.Create;
    try
        ConfigIni.ReadSection ( 'Files', arquivos );

        for i := 0 to arquivos.Count - 1 do
            loadFile ( ConfigIni.ReadString ( 'Files', arquivos[i], '' ) );

        PgcBaseFiles.ActivePageIndex := 0;
    finally
        arquivos.Free;
    end;

    WindowState := wsMaximized;
  end;

procedure TFrmMain.FormCloseQuery ( Sender: TObject; var CanClose: Boolean );
  begin
    canClose := false;
    canClose := saveAll;
  end;

procedure TFrmMain.FormClose ( Sender: TObject; var Action: TCloseAction );
  var i: Integer;
  begin
    ConfigIni.EraseSection ( 'Files' );

    for i := 0 to fileNames.Count - 1 do
        if fileNames[i] <> '' then
            ConfigIni.WriteString ( 'Files', 'File' + IntToStr ( i ), fileNames[i] );

    ConfigIni.UpdateFile;
  end;

procedure TFrmMain.FormResize(Sender: TObject);
  begin
    if keepAspect then
        PgcBaseFiles.Height := Trunc ( ( PgcBaseFiles.Height + PnlGenerated.Height ) * aspect );
  end;

procedure TFrmMain.Splitter1Moved(Sender: TObject);
  begin
    keepAspect := false;
  end;

procedure TFrmMain.MnuNewClick ( Sender: TObject );
  var newMemo: TMemo;
  begin
    newMemo := newPage ( noName );

    fileNames.Add ( noName );
    fileMemos.Add ( newMemo );
  end;

procedure TFrmMain.MnuOpenClick ( Sender: TObject );
  begin
    if OpenDialog.Execute then
        loadFile ( OpenDialog.FileName );
  end;

procedure TFrmMain.MnuCloseClick ( Sender: TObject );
  var page: TTabSheet;
  begin
    saveFile ( PgcBaseFiles.ActivePageIndex );

    fileNames.Delete ( PgcBaseFiles.ActivePageIndex );
    fileMemos.Delete ( PgcBaseFiles.ActivePageIndex );

    page := PgcBaseFiles.ActivePage;
    page.PageControl := nil;
    page.Free;
  end;

procedure TFrmMain.MnuExitClick ( Sender: TObject );
  begin
    Close;
  end;

procedure TFrmMain.MnuCloseTabClick ( Sender: TObject );
  begin
    MnuCloseClick ( sender );
  end;

procedure TFrmMain.MnuGenerateClick ( Sender: TObject );
  var start: TDateTime;
      time: Extended;
  begin
    if fileNames.Count > 0 then
      begin
        saveAll;

        start := Now;

        IBDb.Close;
        IBDb.DatabaseName := ConfigIni.ReadString ( 'Database', 'Path', '' );

        if IBDb.DatabaseName = '' then
            IBDb.Open;
        try
            PageBuilder.HTMLDoc.Assign ( TMemo ( fileMemos[0] ).Lines );
            PBTagInt.IncludeDir := ConfigIni.ReadString ( 'Template', 'Path', '' );

            if PBTagInt.IncludeDir = '' then
                PBTagInt.IncludeDir := ExtractFilePath ( fileNames[0] );

            PBTagInt.Clear;
            DBIbxTagInt.Clear;
            MemoGeneratedHtml.Lines.Text := PageBuilder.Content;
        finally
            IBDb.Close;
        end;

        time := ( Now - start ) * SecsPerDay;
        LblTime.Caption := 'Last "page build" in ' + FormatFloat ( '##0.000', time ) + ' seconds.';
        LblTime.Visible := true;
      end
    else
        raise Exception.Create ( 'Open at last one "html" file.' );
  end;

procedure TFrmMain.MnuPreviewClick(Sender: TObject);
  var genFile: String;
  begin
    MnuGenerate.Click;

    genFile := ChangeFileExt ( fileNames[0], '.generatedHtml' );

    MemoGeneratedHtml.Lines.SaveToFile ( genFile );
    try
        ExecAndWait ( '"' + ConfigIni.ReadString ( 'Preview', 'Command', '' ) + '" "' + genFile + '"', ExtractFilePath ( genFile ) );
        Sleep ( 5000 );    // Wait 5 seconds while ExecAndWait doesn't wait.        
    finally
        DeleteFile ( genFile );
    end;
  end;

procedure TFrmMain.MnuTemplateFontClick ( Sender: TObject );
  var i: Integer;
  begin
    FontDialog.Font.Assign ( templateFont );

    if FontDialog.Execute then
      begin
        templateFont.Assign ( FontDialog.Font );

        for i := 0 to fileMemos.Count - 1 do
            TMemo ( fileMemos[i] ).Font.Assign ( templateFont );
      end;
  end;

procedure TFrmMain.MnuGeneratedFontClick ( Sender: TObject );
  begin
    FontDialog.Font.Assign ( MemoGeneratedHtml.Font );

    if FontDialog.Execute then
        MemoGeneratedHtml.Font.Assign ( FontDialog.Font );
  end;

procedure TFrmMain.MnuConfigClick(Sender: TObject);
  begin
    if FrmConfig = nil then
        FrmConfig := TFrmConfig.Create ( Application );

    FrmConfig.ShowModal;
  end;

procedure TFrmMain.MnuAboutClick ( Sender: TObject );
  begin
    if FrmAbout = nil then
        FrmAbout := TFrmAbout.Create ( Application );

    FrmAbout.ShowModal;
  end;

function TFrmMain.newPage ( tabCaption: String ): TMemo;
  var newPage: TTabSheet;
  begin
    newPage := TTabSheet.Create ( PgcBaseFiles );

    newPage.PageControl := PgcBaseFiles;
    newPage.Caption := tabCaption;

    PgcBaseFiles.ActivePage := newPage;

    Result := TMemo.Create ( Self );

    Result.Parent := PgcBaseFiles.ActivePage;
    Result.Align := alClient;
    Result.ScrollBars := ssAutoBoth;
    Result.WordWrap := false;
    Result.Modified := false;
    Result.Font.Assign ( templateFont );
  end;

procedure TFrmMain.loadFile ( fileName: String );
  var newMemo: TMemo;
      page: TTabSheet;
  begin
    newMemo := newPage ( ExtractFileName ( fileName ) );
    try
        newMemo.Lines.LoadFromFile ( fileName );
        fileNames.Add ( fileName );
        fileMemos.Add ( newMemo );
    except
        on e: Exception do
          begin
            ShowMessage ( 'Error loading ' + fileName + ': ' + e.Message );

            page := PgcBaseFiles.ActivePage;
            page.PageControl := nil;
            page.Free;
          end;
    end;
  end;

procedure TFrmMain.saveFile ( idx: Integer );
  var memo: TMemo;
  begin
    memo := TMemo ( fileMemos [ idx ] );

    if memo.Modified then
        if MessageDlg ( 'Save File', 'File "' + fileNames [ idx ] + '" was changed. Save?', mtConfirmation, mbYesNoCancel, 0, mbYes ) = mrYes then
          begin
            if CompareText ( Copy ( fileNames [ idx ], 1, tNoname ), noName ) = 0 then
              begin
                if SaveDialog.Execute then
                  begin
                    memo.Lines.SaveToFile ( SaveDialog.FileName );
                    memo.Modified := false;
                    fileNames [ idx ] := SaveDialog.FileName;
                    PgcBaseFiles.Pages [ idx ].Caption := ExtractFileName ( SaveDialog.FileName );
                  end
                else
                    Abort;
              end
            else
              begin
                memo.Lines.SaveToFile ( fileNames [ idx ] );
                memo.Modified := false;
              end;
          end
        else
            // Leave file unsaved, but opened.
    else
        // Not changed. Nothing to do.
  end;

function TFrmMain.saveAll: Boolean;
  var i: Integer;
  begin
    for i := 0 to fileNames.Count - 1 do
        saveFile ( i );

    Result := true;
  end;

end.

